#!/usr/bin/perl

#
# genmcs:  Translate <design>.ncd and <out>.bin/.elf files into an .mcs (auto cache-loading)
#

use File::Basename;

$rundir = dirname($0);

$debug = 0;
$progvers = "1.01";
$ncdfile = "";
$binfile = "";
$format = "mcs";
$outfile = "genmcs.mcs";
$defaultpart = "4vfx12ff668";
$startclk = "CCLK";
$usedone = $runbitgen = 1;
$| = 1;

print "\ngenmcs $progvers\n\n";

for ($i=0; ; $i++) {
   $arg = substr($ARGV[$i],0,2);
   if ($arg eq "-x") { 
      $debug = 1;
      next;
   }
   if ($arg eq "-a") { 
      $userarch = $ARGV[$i+1];
      $i++;
      next;
   }
   if ($arg eq "-b") { 
      $runbitgen = 0;
      next;
   }
   if ($arg eq "-c") { 
      $cachefile = $ARGV[$i+1];
      $i++;
      next;
   }
   if ($arg eq "-d") { 
      $usedone = 0;
      next;
   }
   if ($arg eq "-f") { 
      $format = $ARGV[$i+1];
      $i++;
      next;
   }
   if ($arg eq "-p") { 
      $userpart = $ARGV[$i+1];
      $i++;
      next;
   }
   if ($arg eq "-s") { 
      $startclk = $ARGV[$i+1];
      $i++;
      next;
   }
   if ($arg ne "") {
      if ($ncdfile eq "")    { $ncdfile = $ARGV[$i];                                     
                               $ncdfile = $ncdfile.".ncd" if (! ($ncdfile =~ /\.ncd$/i))         }
      elsif ($binfile eq "") { $binfile = $ARGV[$i];                                             }
      else                   { $outfile = $ARGV[$i];
                               $outfile = $outfile.".$format" if (! ($outfile =~ /\.$format$/i)) }
      next;
   }   
   else { last; }
}

if ($#ARGV < 0) { &printhelp; }


#############################
#                           #
#  Create output .bit file  #
#                           #
#############################

$ncdname = substr($ncdfile,0,length($ncdfile)-4);
if ($runbitgen) {
   print "Generating $ncdname.bit...\n";
   $temp = `bitgen -w -b -g Persist:Yes -g Startupclk:$startclk $ncdfile $ncdname.bit`;
   if (($temp =~ /Command not found/i) || ($temp eq "")) {
      print "\nError: bitgen executable not found, could not create $ncdname.bit.\n";
      print "       Make sure ISE setup is loaded.\n";
      &exitroutine;
   }
   if ($temp =~ /ERROR:/i) {
      print "\nError: bitgen aborted with an error, could not create $ncdname.bit.\n";
      &exitroutine;
   }
}


#############################
#                           #
#  Read in design.rbt file  #
#                           #
#############################

$rbtfile = $ncdname.".rbt";
print "Reading $rbtfile...";
open(INFILE, "$rbtfile") || die ("ERROR: $rbtfile cannot be opened\n");
while (<INFILE>) {
   chomp;
   if ($_ =~ /^Design name:\s+(\w+)$/)   { $desname = $1;   next; }
   if ($_ =~ /^Part:\s+(\w+)$/)          { $filepart = $1;  next; }
   if ($_ =~ /^Architecture:\s+(\w+)$/)  { $filearch = $1;  next; }
   if ($_ =~ /^Date:/)                   { next;                  }
   if ($_ =~ /^Bits:\s+(\w+)$/)          { $bitcount1 = $1; next; }
   next if (! ($_ =~ /^[10]/));
   push(@filelines, $_);
}
close(INFILE);
print "(parttype $filepart, architecture $filearch)\n";


##################################
#                                #
#  Convert binary file to ascii  #
#                                #
##################################

print "Converting $binfile...\n\n";
if ($binfile =~ /\.elf$/io) {
   $elffile = $binfile;
   $binfile = substr($binfile,0,length($binfile)-4).".bin";
   $svffile = substr($binfile,0,length($binfile)-4).".svf";
   open(SCRFILE, "> genmcs.scr") || die ("ERROR: genmcs.scr cannot be opened\n");
   select(SCRFILE);
   print "connect ppc hw -cable type xilinx_svffile fname $svffile -configdevice devicenr 1 irlength 4 -debugdevice devicenr 1 icachestartadr 0xffff0000 dcachestartadr 0xffff8000\n";
   print "debugconfig -reset_on_run disable\n";
   print "rwr msr 0x40000\n";
   print "rwr msr 0\n";
   print "dow $elffile\n";
   print "run\n";
   print "exit\n";
   close(SCRFILE);
   my $nx = (`xmd -help` =~ /\-nx/) ? "-nx" : "";
   $temp = `xmd $nx < genmcs.scr`;
#   print "$temp\n";
   $temp = `xilperl $rundir/svf2bin.pl $svffile $binfile`;
#   print "$temp\n";
   $temp = unlink("genmcs.scr", $svffile);
#   print STDOUT "\n";
}

open(BINARYIN, "$binfile") || die ("ERROR: $binfile cannot be opened\n");
binmode(BINARYIN);
while (read(BINARYIN,$byte,1)) {
   $val = unpack("C", $byte);
   $rawbyte = "";
   if ($val & 128) { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   if ($val & 64)  { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   if ($val & 32)  { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   if ($val & 16)  { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   if ($val & 8)   { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   if ($val & 4)   { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   if ($val & 2)   { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   if ($val & 1)   { $rawbyte = $rawbyte."1"; }
   else            { $rawbyte = $rawbyte."0"; }
   push(@rawlines, $rawbyte);
}
close(BINARYIN);
select(STDOUT);


##################################
#                                #
#  Write out combined .rbt file  #
#                                #
##################################

$filearch = $userarch if ($userarch);
$filepart = $userpart if ($userpart);
$filepart = $defaultpart if ($filepart eq "");

$outname = substr($outfile,0,length($outfile)-4);
$rbtfile = $outname.".rbt";
print "Writing $rbtfile, parttype $filepart, architecture $filearch\n";
open (OUTFILE, "> $rbtfile") || die ("ERROR: $rbtfile cannot be opened\n");
select(OUTFILE);

$date = localtime();

$word32count = int(($#rawlines + 4) / 4);
$bitcount2 = 384 + (32 * ($word32count + 1)) + 64;  # header, USR_ACCESS data, NOPs

&printcachedata(0);
close(OUTFILE);

if ($cachefile ne "") {
   open (OUTFILE, "> $cachefile.rbt") || die ("ERROR: $cachefile.rbt cannot be opened\n");
   select(OUTFILE);
   &printcachedata(1);
   close(OUTFILE);
}
select(STDOUT);


#####################################
#                                   #
#  Create .mcs file from .rbt file  #
#                                   #
#####################################

print "Creating $outfile...\n";
$temp = `promgen`;
if (($temp =~ /Command not found/i) || ($temp eq "")) {
   print "\nError: promgen executable not found, could not create $outfile.\n";
   print "       Make sure ISE setup is loaded.\n";
}
else { $temp = `promgen -w -p $format -x xcf32p -u 0 $rbtfile -o $outfile`; }

&exitroutine;


#
#  Start of subroutines
#

sub printcachedata {
   $cacheonly = $_[0];

   print "Xilinx ASCII Bitstream\n";
   if ($cacheonly) {
      print "Created by Bitstream (genmcs $progvers, $binfile cache data only)\n";
   }
   else {
      print "Created by Bitstream (genmcs $progvers, $binfile cache data at end of file)\n";
   }
   print "Design name: 	$ncdname.ncd\n";
   print "Architecture:	$filearch\n";
   print "Part:        	$filepart\n";
   print "Date:        	$date\n";
   if ($cacheonly) {
      print "Bits:        	",$bitcount2,"\n";
   }
   else {
      print "Bits:        	",$bitcount1 + $bitcount2,"\n";
      foreach $line (@filelines) { &printword($line); }
   }

   &printword("11111111111111111111111111111111");
   &printword("10101010100110010101010101100110");
   &printword("00100000000000000000000000000000");
   &printword("00110000000000001000000000000001");  # RCFG command, caused status register = 0?
   &printword("00000000000000000000000000000100");
   &printword("00100000000000000000000000000000");
   &printword("00110000000000000010000000000001");  # Write 0 to FAR, for Type 2 AXSS error
   &printword("00000000000000000000000000000000");
   &printword("00100000000000000000000000000000");
   &printword("00100000000000000000000000000000");
   &printword("00110000000000011010000000000000");  # Type 1 AXSS write, 0 Words

   $rawword = "";
   $val = ($word32count + 1);
   foreach $i (1 .. 27) {
      if ($val & 1)   { $rawword = "1".$rawword; }
      else            { $rawword = "0".$rawword; }
      $val = ($val >> 1);
   }
   $axsscmd = "01010".$rawword;

   &printword($axsscmd);  # Type 2 AXSS write, N Words (27 bits)

   if ($usedone) { 
      $rawword = "";
      $val = $word32count;
      foreach $i (1 .. 27) {
         if ($val & 1)   { $rawword = "1".$rawword; }
         else            { $rawword = "0".$rawword; }
         $val = ($val >> 1);
      }
      $uar_count = "00000".$rawword;
   }
   else { $uar_count = "00000000000000000000000000000000"; }

   &printword($uar_count);

   $i = 0;
   while ($i <= $#rawlines) {
      $word = $rawlines[$i+3].$rawlines[$i+2].$rawlines[$i+1].$rawlines[$i];
      if (length($word) != 32) { 
         $pad = "0" x (32 - length($word));
         $word = $pad.$word;
      }
      &printword($word);
      $i += 4;
   }
   &printword("00100000000000000000000000000000");
   &printword("00100000000000000000000000000000");
}

sub printword {
   $printword = $_[0];

   print substr($printword,0,8);
   print substr($printword,8,8);
   print substr($printword,16,8);
   print substr($printword,24,8), "\n";
}

sub printhelp {
   print "usage: genmcs [options] <design>[.ncd] <data.[bin,elf]> [output][.mcs]\n\n";
   print "       -a ARCH           - Architecture type (default=virtex4)\n\n";
   print "       -b                - Don't run bitgen, .rbt and .bit already present\n\n";
   print "       -c FILENAME       - Write cache .rbt file to FILENAME.rbt (for testing)\n\n";
   print "       -f FORMAT         - Specify output format (default=mcs)\n\n";
   print "       -p PARTTYPE       - Virtex4 parttype in bitstream (default=$defaultpart)\n\n";
   print "       -s CLK            - Specify CLK type for bitgen STARTUPCLK parameter (default=CCLK)\n\n";
   print "       <design.ncd>      - Input design .ncd file\n\n";
   print "       <data.[bin,elf]>  - Input binary file\n\n";
   print "       [output.mcs]      - Output .mcs file (default=\"$outfile\")\n\n";
   exit(1);
}

sub exitroutine {
   print "\n";
   exit(0);
}

