#!/usr/bin/perl

#
# svf2bin:  Translate a SVF file to a binary file.
#

$debug = 0;
$progvers = "0.90";
$svffile = "";
$binfile = "executable.bin";
$| = 1;

print "\nsvf2bin $progvers\n\n";

for ($i=0; ; $i++) {
   $arg = substr($ARGV[$i],0,2);
   if ($arg eq "-x") { 
      $debug = 1;
      next;
   }
   if ($arg ne "") {
      if ($svffile eq "")    { 
	  $svffile = $ARGV[$i];                                     
	  $svffile = $svffile.".svf" if (! ($svffile =~ /\.svf$/i)) }
      else                   {
	  $binfile = $ARGV[$i];
	  $binfile = $binfile.".bin" if (! ($binfile =~ /\.bin$/i)) }
      next;
   }   
   else { last; }
}

if ($#ARGV < 0) { &printhelp; }

####################
#                  #
#  Read in svffile #
#                  #
####################

print "Reading $svffile...\n";
open(INFILE, "$svffile") || die ("ERROR: $svffile cannot be opened\n");
while (<INFILE>) {
    chomp;
    push(@filelines, $_);
}
close(INFILE);

print "Writing $binfile...\n";

open(BINARYOUT, "> $binfile") || die ("ERROR: $binfile cannot be opened\n");
binmode(BINARYOUT);
select(BINARYOUT);

#set global variables for HIR, HDR, TIR, TDR
$hir_len = $hdr_len = $tir_len = $tdr_len = 0;
$hir_bin = $hdr_bin = $tir_bin = $tdr_bin = 0;

# print the synchronization header: 0xff 0xfc
for (my $i=0; $i < 7; $i++) {&setbits(1,1)};
&setbits(0,0);

foreach (@filelines) {
    chomp;
    if ($_ =~ /^STATE RESET/)                                {
	&print_state_reset;
	next;
    }
    if ($_ =~ /^STATE IDLE/)                                 { next; }
    if ($_ =~ /^SIR\s+(\d+)\s+TDI\s+\((\p{IsXDigit}+)\)/)    {
	&print_sxr(1,$1,$2);
	next;
    }
    if ($_ =~ /^SDR\s+(\d+)\s+TDI\s+\((\p{IsXDigit}+)\)/)    {
	&print_sxr(0,$1,$2);
	next;
    }
    if ($_ =~ /^HIR\s+(\d+)/)    {
	$hir_len = $1;
	$hir_bin = set_xxR($1, $_);
	next;
    }
    if ($_ =~ /^HDR\s+(\d+)/)    {
	$hdr_len = $1;
	$hdr_bin = set_xxR($1, $_);
	next;
    }
    if ($_ =~ /^TIR\s+(\d+)/) {
	$tir_len = $1;
	$tir_bin = set_xxR($1, $_);
	next;
    }
    if ($_ =~ /^TDR\s+(\d+)/)    {
	$tdr_len = $1;
	$tdr_bin = set_xxR($1, $_);
	next;
    }
    die("ERROR: I do not know what to do with\n    $_\n");
}

#make sure the last byte gets flushed out
for (my $i=0; $i < 4; $i++) {&setbits(0,0)};

close(BINARYOUT);
select(STDOUT);

&exitroutine;


#
#  Start of subroutines
#

# convert hex string to reversed binary string
sub hex2bin {
    my $hex     = $_[0];
    my $numbits = $_[1];
    my $bin     = "";

    for ($i=0; $i < length($hex); $i++) {
	$bin = $bin.unpack("B4", pack("H", substr($hex,$i,1)));
    }
    return substr(reverse($bin), 0, $numbits);
}

sub BEGIN {
    use integer;

    my $bitcount = 0;
    my $byteval  = 0;

    sub setbits {
        my $tck = $_[0];
        my $tdi = $_[1];

	$byteval = ($byteval << 1) | $tck;
	$byteval = ($byteval << 1) | $tdi;
	$bitcount += 2;

	if ($bitcount == 8) {
	    print pack("C", $byteval);
	    $bitcount = 0;
	    $byteval = 0;
	}
    }
}

sub print_state_reset {
    &setbits(1,0);
    &setbits(1,0);
    &setbits(1,0);
    &setbits(1,0);
    &setbits(1,0);
    &setbits(0,0);   # go to state idle
}

sub print_sxr {
    my $isSIR   = $_[0];
    my $numbits = $_[1];
    my $hex     = $_[2];
    my $bin     = hex2bin($hex, $numbits);

    # add header and trailer
    if ($isSIR) {
	$numbits += $sir_len + $tir_len;
	$bin     =  $sir_bin.$bin.$tir_bin;
    } else {
	$numbits += $sdr_len + $tdr_len;
	$bin     =  $sdr_bin.$bin.$tdr_bin;
    }

    # get into SIR/SDR
    if ($isSIR) {&setbits(1,0)};
    &setbits(1,0);
    &setbits(0,0);
    &setbits(0,0);

    # shift bits
    for (my $i=0; $i < $numbits-1; $i++) {
	&setbits(0,substr($bin,$i,1));
    }
    &setbits(1,substr($bin,$numbits-1,1));

    # get back to IDLE
    &setbits(1,0);
    &setbits(0,0);
}

sub set_xxR {
    my $numbits = $_[0];
    my $line    = $_[1];
    my $bin     = "";

    if (($numbits > 0) && ($line =~ /\((\p{IsXDigit}+)\)/)) {
	$bin = hex2bin($1, $numbits);
    } elsif ($numbits > 0) {
	die("ERROR: Syntax error in\n    $line\n");
    }
    return $bin;
}

sub printhelp {
    print "Convert a SVF file to a binary file consisting of [TCK,TDI] bit pairs\n";
    print "usage: svf2bin <input>[.svf] [<output.[bin]]>\n\n";
    
    print "       <input.svf>       - Input .svf file\n\n";
    print "       [<output.bin>]    - Output .bin file (default executable.bin)\n\n";
    
    exit(1);
}

sub exitroutine {
    print "\n";
    exit(0);
}


# Implementation Notes:

# aa: 10       10      10      10
# 8a: 10 RESET 00 IDLE 10 SDRS 10 SIRS
# 04: 00 CIR   00 SIR  01      00
# 68: 01       10 EIR  10 UIR  00 IDLE
# 80: 10 SDRS  00 CDR  00 SDR  00
# 00: 00       00      00      00
# 00: 00       00      00      00
# 00: 00       00      00      00
# 00: 00       00      00      00
# 00: 00       00      00      00
# 00: 00       00      00      00
# 00: 00       00      00      00
# 07: 00       00      01      11 EDR
# 8a: 10 UDR   00 IDLE 10 SDRS 10 SIRS
# 04: 00 CIR   00 SIR  01      00
# 68: 01       10 EIR  10 UIR  00 IDLE

# STATE RESET:       10 10 10 10 10 00
# STATE IDLE:        ignore
# HIR/TIR/HDR/TDR 0: ignore
# HIR/TIR/HDR/TDR ?: error
# SIR 4  TDI (wxyz): 10 10 00 00 0z 0y 0x 1w 10 00
# SDR 33 TDI ():     10 00 00 0z 0y 0y 0y 0y ... 1w 10 00
