//-----------------------------------------------------------------------------
// uc2.v
//-----------------------------------------------------------------------------

module uc2
  (
    sys_clk,
    sys_rst,
    gpio_in,
    gpio_out,
    interrupt,
    sys_rst_out,
    jtag_tck,
    jtag_tms,
    jtag_tdi,
    jtag_tdo,
    jtag_tdoen,
    DSOCMBRAMWRDBUS,
    DSOCMRDADDRVALID,
    DSOCMWRADDRVALID,
    tx_fifo_full,
    proc_clk
  );
  input sys_clk;
  input sys_rst;
  input [0:31] gpio_in;
  output [0:31] gpio_out;
  input interrupt;
  output sys_rst_out;
  input jtag_tck;
  input jtag_tms;
  input jtag_tdi;
  output jtag_tdo;
  output jtag_tdoen;
  output [0:31] DSOCMBRAMWRDBUS;
  output DSOCMRDADDRVALID;
  output DSOCMWRADDRVALID;
  input tx_fifo_full;
  input proc_clk;

  // Internal signals

  wire C405DBGMSRWE;
  wire [0:0] addr_ff_0;
  wire dbgc405debughalt;
  wire icu_req_to_addrack;
  wire net_gnd0;
  wire [0:0] net_gnd1;
  wire [0:1] net_gnd2;
  wire [0:2] net_gnd3;
  wire [0:3] net_gnd4;
  wire [0:31] net_gnd32;
  wire [0:63] net_gnd64;
  wire net_vcc0;
  wire [0:3] pgassign1;
  wire [0:63] pgassign2;
  wire [0:1] pgassign3;
  wire [0:7] pgassign4;
  wire [0:7] pgassign5;
  wire [0:63] pgassign6;
  wire [0:7] pgassign7;
  wire [0:0] pgassign8;
  wire proc_sys_reset_0_Rstc405resetsys;
  wire sys_clk_in;
  wire system_reset_in;
  wire system_reset_req_from_ppc405;
  wire tck;
  wire tdi;
  wire tdo;
  wire tdoen;
  wire tms;
  wire [0:0] wdaddr;

  // Internal assignments

  assign sys_clk_in = sys_clk;
  assign system_reset_in = sys_rst;
  assign sys_rst_out = system_reset_req_from_ppc405;
  assign tck = jtag_tck;
  assign tms = jtag_tms;
  assign tdi = jtag_tdi;
  assign jtag_tdo = tdo;
  assign jtag_tdoen = tdoen;
  assign pgassign2[0:63] = 64'h4bff3ffa38000001;
  assign pgassign3[0:1] = 2'b01;
  assign pgassign4[0:7] = 8'h01;
  assign pgassign5[0:7] = 8'h83;
  assign pgassign6[0:63] = 64'h7C1BFBA64BFF3FFE;
  assign pgassign7[0:7] = 8'h00;
  assign pgassign8[0:0] = 1'b0;
  assign pgassign1[0:0] = 1'b0;
  assign pgassign1[1:1] = 1'b0;
  assign pgassign1[2:2] = wdaddr[0:0];
  assign pgassign1[3:3] = 1'b0;
  assign net_gnd0 = 1'b0;
  assign net_gnd1[0:0] = 1'b0;
  assign net_gnd2[0:1] = 2'b00;
  assign net_gnd3[0:2] = 3'b000;
  assign net_gnd32[0:31] = 32'b00000000000000000000000000000000;
  assign net_gnd4[0:3] = 4'b0000;
  assign net_gnd64[0:63] = 64'b0000000000000000000000000000000000000000000000000000000000000000;
  assign net_vcc0 = 1'b1;

  debughalt_0_wrapper
    debughalt_0 (
      .CPU_HALT_N ( net_vcc0 ),
      .CPU_HALT ( tx_fifo_full ),
      .DEASSERT_HALT ( C405DBGMSRWE ),
      .ASSERT_HALT ( net_gnd0 ),
      .DEBUGHALT_HALT ( dbgc405debughalt )
    );

  vl_0_wrapper
    vl_0 (
      .Op1 ( wdaddr[0:0] ),
      .Op2 ( net_gnd1[0:0] ),
      .Res ( addr_ff_0[0:0] )
    );

  ff_0_wrapper
    ff_0 (
      .Clk ( sys_clk_in ),
      .RST ( net_gnd0 ),
      .SET ( net_gnd0 ),
      .CE ( net_gnd0 ),
      .D ( addr_ff_0[0:0] ),
      .Q ( wdaddr[0:0] )
    );

  ppc405_0_wrapper
    ppc405_0 (
      .C405CPMCORESLEEPREQ (  ),
      .C405CPMMSRCE (  ),
      .C405CPMMSREE (  ),
      .C405CPMTIMERIRQ (  ),
      .C405CPMTIMERRESETREQ (  ),
      .C405XXXMACHINECHECK (  ),
      .CPMC405CLOCK ( proc_clk ),
      .CPMC405CORECLKINACTIVE ( net_gnd0 ),
      .CPMC405CPUCLKEN ( net_vcc0 ),
      .CPMC405JTAGCLKEN ( net_vcc0 ),
      .CPMC405TIMERCLKEN ( net_vcc0 ),
      .CPMC405TIMERTICK ( net_vcc0 ),
      .MCBCPUCLKEN ( net_vcc0 ),
      .MCBTIMEREN ( net_vcc0 ),
      .MCPPCRST ( net_vcc0 ),
      .PLBCLK ( sys_clk_in ),
      .CPMDCRCLK ( net_vcc0 ),
      .CPMFCMCLK ( net_vcc0 ),
      .C405RSTCHIPRESETREQ (  ),
      .C405RSTCORERESETREQ (  ),
      .C405RSTSYSRESETREQ ( system_reset_req_from_ppc405 ),
      .RSTC405RESETCHIP ( proc_sys_reset_0_Rstc405resetsys ),
      .RSTC405RESETCORE ( proc_sys_reset_0_Rstc405resetsys ),
      .RSTC405RESETSYS ( proc_sys_reset_0_Rstc405resetsys ),
      .APUFCMDECODED (  ),
      .APUFCMDECUDI (  ),
      .APUFCMDECUDIVALID (  ),
      .APUFCMENDIAN (  ),
      .APUFCMFLUSH (  ),
      .APUFCMINSTRUCTION (  ),
      .APUFCMINSTRVALID (  ),
      .APUFCMLOADBYTEEN (  ),
      .APUFCMLOADDATA (  ),
      .APUFCMLOADDVALID (  ),
      .APUFCMOPERANDVALID (  ),
      .APUFCMRADATA (  ),
      .APUFCMRBDATA (  ),
      .APUFCMWRITEBACKOK (  ),
      .APUFCMXERCA (  ),
      .FCMAPUCR ( net_gnd4 ),
      .FCMAPUDCDCREN ( net_gnd0 ),
      .FCMAPUDCDFORCEALIGN ( net_gnd0 ),
      .FCMAPUDCDFORCEBESTEERING ( net_gnd0 ),
      .FCMAPUDCDFPUOP ( net_gnd0 ),
      .FCMAPUDCDGPRWRITE ( net_gnd0 ),
      .FCMAPUDCDLDSTBYTE ( net_gnd0 ),
      .FCMAPUDCDLDSTDW ( net_gnd0 ),
      .FCMAPUDCDLDSTHW ( net_gnd0 ),
      .FCMAPUDCDLDSTQW ( net_gnd0 ),
      .FCMAPUDCDLDSTWD ( net_gnd0 ),
      .FCMAPUDCDLOAD ( net_gnd0 ),
      .FCMAPUDCDPRIVOP ( net_gnd0 ),
      .FCMAPUDCDRAEN ( net_gnd0 ),
      .FCMAPUDCDRBEN ( net_gnd0 ),
      .FCMAPUDCDSTORE ( net_gnd0 ),
      .FCMAPUDCDTRAPBE ( net_gnd0 ),
      .FCMAPUDCDTRAPLE ( net_gnd0 ),
      .FCMAPUDCDUPDATE ( net_gnd0 ),
      .FCMAPUDCDXERCAEN ( net_gnd0 ),
      .FCMAPUDCDXEROVEN ( net_gnd0 ),
      .FCMAPUDECODEBUSY ( net_gnd0 ),
      .FCMAPUDONE ( net_gnd0 ),
      .FCMAPUEXCEPTION ( net_gnd0 ),
      .FCMAPUEXEBLOCKINGMCO ( net_gnd0 ),
      .FCMAPUEXECRFIELD ( net_gnd3 ),
      .FCMAPUEXENONBLOCKINGMCO ( net_gnd0 ),
      .FCMAPUINSTRACK ( net_gnd0 ),
      .FCMAPULOADWAIT ( net_gnd0 ),
      .FCMAPURESULT ( net_gnd32 ),
      .FCMAPURESULTVALID ( net_gnd0 ),
      .FCMAPUSLEEPNOTREADY ( net_gnd0 ),
      .FCMAPUXERCA ( net_gnd0 ),
      .FCMAPUXEROV ( net_gnd0 ),
      .C405PLBICUABUS (  ),
      .C405PLBICUBE (  ),
      .C405PLBICURNW (  ),
      .C405PLBICUABORT (  ),
      .C405PLBICUBUSLOCK (  ),
      .C405PLBICUU0ATTR (  ),
      .C405PLBICUGUARDED (  ),
      .C405PLBICULOCKERR (  ),
      .C405PLBICUMSIZE (  ),
      .C405PLBICUORDERED (  ),
      .C405PLBICUPRIORITY (  ),
      .C405PLBICURDBURST (  ),
      .C405PLBICUREQUEST ( icu_req_to_addrack ),
      .C405PLBICUSIZE (  ),
      .C405PLBICUTYPE (  ),
      .C405PLBICUWRBURST (  ),
      .C405PLBICUWRDBUS (  ),
      .C405PLBICUCACHEABLE (  ),
      .PLBC405ICUADDRACK ( icu_req_to_addrack ),
      .PLBC405ICUBUSY ( net_gnd0 ),
      .PLBC405ICUERR ( net_gnd0 ),
      .PLBC405ICURDBTERM ( net_gnd0 ),
      .PLBC405ICURDDACK ( net_vcc0 ),
      .PLBC405ICURDDBUS ( pgassign2 ),
      .PLBC405ICURDWDADDR ( pgassign1 ),
      .PLBC405ICUREARBITRATE ( net_gnd0 ),
      .PLBC405ICUWRBTERM ( net_gnd0 ),
      .PLBC405ICUWRDACK ( net_gnd0 ),
      .PLBC405ICUSSIZE ( pgassign3 ),
      .PLBC405ICUSERR ( net_gnd0 ),
      .PLBC405ICUSBUSYS ( net_gnd0 ),
      .C405PLBDCUABUS (  ),
      .C405PLBDCUBE (  ),
      .C405PLBDCURNW (  ),
      .C405PLBDCUABORT (  ),
      .C405PLBDCUBUSLOCK (  ),
      .C405PLBDCUU0ATTR (  ),
      .C405PLBDCUGUARDED (  ),
      .C405PLBDCULOCKERR (  ),
      .C405PLBDCUMSIZE (  ),
      .C405PLBDCUORDERED (  ),
      .C405PLBDCUPRIORITY (  ),
      .C405PLBDCURDBURST (  ),
      .C405PLBDCUREQUEST (  ),
      .C405PLBDCUSIZE (  ),
      .C405PLBDCUTYPE (  ),
      .C405PLBDCUWRBURST (  ),
      .C405PLBDCUWRDBUS (  ),
      .C405PLBDCUCACHEABLE (  ),
      .C405PLBDCUWRITETHRU (  ),
      .PLBC405DCUADDRACK ( net_gnd0 ),
      .PLBC405DCUBUSY ( net_gnd0 ),
      .PLBC405DCUERR ( net_gnd0 ),
      .PLBC405DCURDBTERM ( net_gnd0 ),
      .PLBC405DCURDDACK ( net_gnd0 ),
      .PLBC405DCURDDBUS ( net_gnd64 ),
      .PLBC405DCURDWDADDR ( net_gnd4 ),
      .PLBC405DCUREARBITRATE ( net_gnd0 ),
      .PLBC405DCUWRBTERM ( net_gnd0 ),
      .PLBC405DCUWRDACK ( net_gnd0 ),
      .PLBC405DCUSSIZE ( net_gnd2 ),
      .PLBC405DCUSERR ( net_gnd0 ),
      .PLBC405DCUSBUSYS ( net_gnd0 ),
      .BRAMDSOCMCLK ( sys_clk_in ),
      .BRAMDSOCMRDDBUS ( gpio_in ),
      .DSARCVALUE ( pgassign4 ),
      .DSCNTLVALUE ( pgassign5 ),
      .DSOCMBRAMABUS (  ),
      .DSOCMBRAMBYTEWRITE (  ),
      .DSOCMBRAMEN (  ),
      .DSOCMBRAMWRDBUS ( DSOCMBRAMWRDBUS ),
      .DSOCMBUSY (  ),
      .DSOCMRDADDRVALID ( DSOCMRDADDRVALID ),
      .DSOCMWRADDRVALID ( DSOCMWRADDRVALID ),
      .DSOCMRWCOMPLETE ( net_vcc0 ),
      .BRAMISOCMCLK ( sys_clk_in ),
      .BRAMISOCMRDDBUS ( pgassign6 ),
      .BRAMISOCMDCRRDDBUS ( net_gnd32 ),
      .ISARCVALUE ( pgassign7 ),
      .ISCNTLVALUE ( pgassign5 ),
      .ISOCMBRAMEN (  ),
      .ISOCMBRAMEVENWRITEEN (  ),
      .ISOCMBRAMODDWRITEEN (  ),
      .ISOCMBRAMRDABUS (  ),
      .ISOCMBRAMWRABUS (  ),
      .ISOCMBRAMWRDBUS ( gpio_out ),
      .ISOCMDCRBRAMEVENEN (  ),
      .ISOCMDCRBRAMODDEN (  ),
      .ISOCMDCRBRAMRDSELECT (  ),
      .DCREMACABUS (  ),
      .DCREMACCLK (  ),
      .DCREMACDBUS (  ),
      .DCREMACENABLER (  ),
      .DCREMACREAD (  ),
      .DCREMACWRITE (  ),
      .EMACDCRACK ( net_gnd0 ),
      .EMACDCRDBUS ( net_gnd32 ),
      .EXTDCRABUS (  ),
      .EXTDCRDBUSOUT (  ),
      .EXTDCRREAD (  ),
      .EXTDCRWRITE (  ),
      .EXTDCRACK ( net_gnd0 ),
      .EXTDCRDBUSIN ( net_gnd32 ),
      .EICC405CRITINPUTIRQ ( net_gnd0 ),
      .EICC405EXTINPUTIRQ ( interrupt ),
      .C405JTGCAPTUREDR (  ),
      .C405JTGEXTEST (  ),
      .C405JTGPGMOUT (  ),
      .C405JTGSHIFTDR (  ),
      .C405JTGTDO ( tdo ),
      .C405JTGTDOEN ( tdoen ),
      .C405JTGUPDATEDR (  ),
      .MCBJTAGEN ( net_vcc0 ),
      .JTGC405BNDSCANTDO ( net_gnd0 ),
      .JTGC405TCK ( tck ),
      .JTGC405TDI ( tdi ),
      .JTGC405TMS ( tms ),
      .JTGC405TRSTNEG ( net_vcc0 ),
      .C405DBGMSRWE ( C405DBGMSRWE ),
      .C405DBGSTOPACK (  ),
      .C405DBGWBCOMPLETE (  ),
      .C405DBGWBFULL (  ),
      .C405DBGWBIAR (  ),
      .DBGC405DEBUGHALT ( dbgc405debughalt ),
      .DBGC405EXTBUSHOLDACK ( net_gnd0 ),
      .DBGC405UNCONDDEBUGEVENT ( net_gnd0 ),
      .C405DBGLOADDATAONAPUDBUS (  ),
      .C405TRCCYCLE (  ),
      .C405TRCEVENEXECUTIONSTATUS (  ),
      .C405TRCODDEXECUTIONSTATUS (  ),
      .C405TRCTRACESTATUS (  ),
      .C405TRCTRIGGEREVENTOUT (  ),
      .C405TRCTRIGGEREVENTTYPE (  ),
      .TRCC405TRACEDISABLE ( net_gnd0 ),
      .TRCC405TRIGGEREVENTIN ( net_gnd0 )
    );

  uc2_reset_0_wrapper
    uc2_reset_0 (
      .UC2_sync_clk ( proc_clk ),
      .Ext_Reset_In ( system_reset_in ),
      .Core_Reset_Req ( net_gnd0 ),
      .Chip_Reset_Req ( net_gnd0 ),
      .System_Reset_Req ( system_reset_req_from_ppc405 ),
      .Reset405out ( proc_sys_reset_0_Rstc405resetsys )
    );

endmodule

module debughalt_0_wrapper
  (
    CPU_HALT_N,
    CPU_HALT,
    DEASSERT_HALT,
    ASSERT_HALT,
    DEBUGHALT_HALT
  );
  input CPU_HALT_N;
  input CPU_HALT;
  input DEASSERT_HALT;
  input ASSERT_HALT;
  output DEBUGHALT_HALT;
endmodule

// synthesis attribute box_type of debughalt_0_wrapper is "black_box";

module vl_0_wrapper
  (
    Op1,
    Op2,
    Res
  );
  input [0:0] Op1;
  input [0:0] Op2;
  output [0:0] Res;
endmodule

// synthesis attribute box_type of vl_0_wrapper is "black_box";

module ff_0_wrapper
  (
    Clk,
    RST,
    SET,
    CE,
    D,
    Q
  );
  input Clk;
  input RST;
  input SET;
  input CE;
  input [0:0] D;
  output [0:0] Q;
endmodule

// synthesis attribute box_type of ff_0_wrapper is "black_box";

module ppc405_0_wrapper
  (
    C405CPMCORESLEEPREQ,
    C405CPMMSRCE,
    C405CPMMSREE,
    C405CPMTIMERIRQ,
    C405CPMTIMERRESETREQ,
    C405XXXMACHINECHECK,
    CPMC405CLOCK,
    CPMC405CORECLKINACTIVE,
    CPMC405CPUCLKEN,
    CPMC405JTAGCLKEN,
    CPMC405TIMERCLKEN,
    CPMC405TIMERTICK,
    MCBCPUCLKEN,
    MCBTIMEREN,
    MCPPCRST,
    PLBCLK,
    CPMDCRCLK,
    CPMFCMCLK,
    C405RSTCHIPRESETREQ,
    C405RSTCORERESETREQ,
    C405RSTSYSRESETREQ,
    RSTC405RESETCHIP,
    RSTC405RESETCORE,
    RSTC405RESETSYS,
    APUFCMDECODED,
    APUFCMDECUDI,
    APUFCMDECUDIVALID,
    APUFCMENDIAN,
    APUFCMFLUSH,
    APUFCMINSTRUCTION,
    APUFCMINSTRVALID,
    APUFCMLOADBYTEEN,
    APUFCMLOADDATA,
    APUFCMLOADDVALID,
    APUFCMOPERANDVALID,
    APUFCMRADATA,
    APUFCMRBDATA,
    APUFCMWRITEBACKOK,
    APUFCMXERCA,
    FCMAPUCR,
    FCMAPUDCDCREN,
    FCMAPUDCDFORCEALIGN,
    FCMAPUDCDFORCEBESTEERING,
    FCMAPUDCDFPUOP,
    FCMAPUDCDGPRWRITE,
    FCMAPUDCDLDSTBYTE,
    FCMAPUDCDLDSTDW,
    FCMAPUDCDLDSTHW,
    FCMAPUDCDLDSTQW,
    FCMAPUDCDLDSTWD,
    FCMAPUDCDLOAD,
    FCMAPUDCDPRIVOP,
    FCMAPUDCDRAEN,
    FCMAPUDCDRBEN,
    FCMAPUDCDSTORE,
    FCMAPUDCDTRAPBE,
    FCMAPUDCDTRAPLE,
    FCMAPUDCDUPDATE,
    FCMAPUDCDXERCAEN,
    FCMAPUDCDXEROVEN,
    FCMAPUDECODEBUSY,
    FCMAPUDONE,
    FCMAPUEXCEPTION,
    FCMAPUEXEBLOCKINGMCO,
    FCMAPUEXECRFIELD,
    FCMAPUEXENONBLOCKINGMCO,
    FCMAPUINSTRACK,
    FCMAPULOADWAIT,
    FCMAPURESULT,
    FCMAPURESULTVALID,
    FCMAPUSLEEPNOTREADY,
    FCMAPUXERCA,
    FCMAPUXEROV,
    C405PLBICUABUS,
    C405PLBICUBE,
    C405PLBICURNW,
    C405PLBICUABORT,
    C405PLBICUBUSLOCK,
    C405PLBICUU0ATTR,
    C405PLBICUGUARDED,
    C405PLBICULOCKERR,
    C405PLBICUMSIZE,
    C405PLBICUORDERED,
    C405PLBICUPRIORITY,
    C405PLBICURDBURST,
    C405PLBICUREQUEST,
    C405PLBICUSIZE,
    C405PLBICUTYPE,
    C405PLBICUWRBURST,
    C405PLBICUWRDBUS,
    C405PLBICUCACHEABLE,
    PLBC405ICUADDRACK,
    PLBC405ICUBUSY,
    PLBC405ICUERR,
    PLBC405ICURDBTERM,
    PLBC405ICURDDACK,
    PLBC405ICURDDBUS,
    PLBC405ICURDWDADDR,
    PLBC405ICUREARBITRATE,
    PLBC405ICUWRBTERM,
    PLBC405ICUWRDACK,
    PLBC405ICUSSIZE,
    PLBC405ICUSERR,
    PLBC405ICUSBUSYS,
    C405PLBDCUABUS,
    C405PLBDCUBE,
    C405PLBDCURNW,
    C405PLBDCUABORT,
    C405PLBDCUBUSLOCK,
    C405PLBDCUU0ATTR,
    C405PLBDCUGUARDED,
    C405PLBDCULOCKERR,
    C405PLBDCUMSIZE,
    C405PLBDCUORDERED,
    C405PLBDCUPRIORITY,
    C405PLBDCURDBURST,
    C405PLBDCUREQUEST,
    C405PLBDCUSIZE,
    C405PLBDCUTYPE,
    C405PLBDCUWRBURST,
    C405PLBDCUWRDBUS,
    C405PLBDCUCACHEABLE,
    C405PLBDCUWRITETHRU,
    PLBC405DCUADDRACK,
    PLBC405DCUBUSY,
    PLBC405DCUERR,
    PLBC405DCURDBTERM,
    PLBC405DCURDDACK,
    PLBC405DCURDDBUS,
    PLBC405DCURDWDADDR,
    PLBC405DCUREARBITRATE,
    PLBC405DCUWRBTERM,
    PLBC405DCUWRDACK,
    PLBC405DCUSSIZE,
    PLBC405DCUSERR,
    PLBC405DCUSBUSYS,
    BRAMDSOCMCLK,
    BRAMDSOCMRDDBUS,
    DSARCVALUE,
    DSCNTLVALUE,
    DSOCMBRAMABUS,
    DSOCMBRAMBYTEWRITE,
    DSOCMBRAMEN,
    DSOCMBRAMWRDBUS,
    DSOCMBUSY,
    DSOCMRDADDRVALID,
    DSOCMWRADDRVALID,
    DSOCMRWCOMPLETE,
    BRAMISOCMCLK,
    BRAMISOCMRDDBUS,
    BRAMISOCMDCRRDDBUS,
    ISARCVALUE,
    ISCNTLVALUE,
    ISOCMBRAMEN,
    ISOCMBRAMEVENWRITEEN,
    ISOCMBRAMODDWRITEEN,
    ISOCMBRAMRDABUS,
    ISOCMBRAMWRABUS,
    ISOCMBRAMWRDBUS,
    ISOCMDCRBRAMEVENEN,
    ISOCMDCRBRAMODDEN,
    ISOCMDCRBRAMRDSELECT,
    DCREMACABUS,
    DCREMACCLK,
    DCREMACDBUS,
    DCREMACENABLER,
    DCREMACREAD,
    DCREMACWRITE,
    EMACDCRACK,
    EMACDCRDBUS,
    EXTDCRABUS,
    EXTDCRDBUSOUT,
    EXTDCRREAD,
    EXTDCRWRITE,
    EXTDCRACK,
    EXTDCRDBUSIN,
    EICC405CRITINPUTIRQ,
    EICC405EXTINPUTIRQ,
    C405JTGCAPTUREDR,
    C405JTGEXTEST,
    C405JTGPGMOUT,
    C405JTGSHIFTDR,
    C405JTGTDO,
    C405JTGTDOEN,
    C405JTGUPDATEDR,
    MCBJTAGEN,
    JTGC405BNDSCANTDO,
    JTGC405TCK,
    JTGC405TDI,
    JTGC405TMS,
    JTGC405TRSTNEG,
    C405DBGMSRWE,
    C405DBGSTOPACK,
    C405DBGWBCOMPLETE,
    C405DBGWBFULL,
    C405DBGWBIAR,
    DBGC405DEBUGHALT,
    DBGC405EXTBUSHOLDACK,
    DBGC405UNCONDDEBUGEVENT,
    C405DBGLOADDATAONAPUDBUS,
    C405TRCCYCLE,
    C405TRCEVENEXECUTIONSTATUS,
    C405TRCODDEXECUTIONSTATUS,
    C405TRCTRACESTATUS,
    C405TRCTRIGGEREVENTOUT,
    C405TRCTRIGGEREVENTTYPE,
    TRCC405TRACEDISABLE,
    TRCC405TRIGGEREVENTIN
  );
  output C405CPMCORESLEEPREQ;
  output C405CPMMSRCE;
  output C405CPMMSREE;
  output C405CPMTIMERIRQ;
  output C405CPMTIMERRESETREQ;
  output C405XXXMACHINECHECK;
  input CPMC405CLOCK;
  input CPMC405CORECLKINACTIVE;
  input CPMC405CPUCLKEN;
  input CPMC405JTAGCLKEN;
  input CPMC405TIMERCLKEN;
  input CPMC405TIMERTICK;
  input MCBCPUCLKEN;
  input MCBTIMEREN;
  input MCPPCRST;
  input PLBCLK;
  input CPMDCRCLK;
  input CPMFCMCLK;
  output C405RSTCHIPRESETREQ;
  output C405RSTCORERESETREQ;
  output C405RSTSYSRESETREQ;
  input RSTC405RESETCHIP;
  input RSTC405RESETCORE;
  input RSTC405RESETSYS;
  output APUFCMDECODED;
  output [0:2] APUFCMDECUDI;
  output APUFCMDECUDIVALID;
  output APUFCMENDIAN;
  output APUFCMFLUSH;
  output [0:31] APUFCMINSTRUCTION;
  output APUFCMINSTRVALID;
  output [0:3] APUFCMLOADBYTEEN;
  output [0:31] APUFCMLOADDATA;
  output APUFCMLOADDVALID;
  output APUFCMOPERANDVALID;
  output [0:31] APUFCMRADATA;
  output [0:31] APUFCMRBDATA;
  output APUFCMWRITEBACKOK;
  output APUFCMXERCA;
  input [0:3] FCMAPUCR;
  input FCMAPUDCDCREN;
  input FCMAPUDCDFORCEALIGN;
  input FCMAPUDCDFORCEBESTEERING;
  input FCMAPUDCDFPUOP;
  input FCMAPUDCDGPRWRITE;
  input FCMAPUDCDLDSTBYTE;
  input FCMAPUDCDLDSTDW;
  input FCMAPUDCDLDSTHW;
  input FCMAPUDCDLDSTQW;
  input FCMAPUDCDLDSTWD;
  input FCMAPUDCDLOAD;
  input FCMAPUDCDPRIVOP;
  input FCMAPUDCDRAEN;
  input FCMAPUDCDRBEN;
  input FCMAPUDCDSTORE;
  input FCMAPUDCDTRAPBE;
  input FCMAPUDCDTRAPLE;
  input FCMAPUDCDUPDATE;
  input FCMAPUDCDXERCAEN;
  input FCMAPUDCDXEROVEN;
  input FCMAPUDECODEBUSY;
  input FCMAPUDONE;
  input FCMAPUEXCEPTION;
  input FCMAPUEXEBLOCKINGMCO;
  input [0:2] FCMAPUEXECRFIELD;
  input FCMAPUEXENONBLOCKINGMCO;
  input FCMAPUINSTRACK;
  input FCMAPULOADWAIT;
  input [0:31] FCMAPURESULT;
  input FCMAPURESULTVALID;
  input FCMAPUSLEEPNOTREADY;
  input FCMAPUXERCA;
  input FCMAPUXEROV;
  output [0:31] C405PLBICUABUS;
  output [0:7] C405PLBICUBE;
  output C405PLBICURNW;
  output C405PLBICUABORT;
  output C405PLBICUBUSLOCK;
  output C405PLBICUU0ATTR;
  output C405PLBICUGUARDED;
  output C405PLBICULOCKERR;
  output [0:1] C405PLBICUMSIZE;
  output C405PLBICUORDERED;
  output [0:1] C405PLBICUPRIORITY;
  output C405PLBICURDBURST;
  output C405PLBICUREQUEST;
  output [0:3] C405PLBICUSIZE;
  output [0:2] C405PLBICUTYPE;
  output C405PLBICUWRBURST;
  output [0:63] C405PLBICUWRDBUS;
  output C405PLBICUCACHEABLE;
  input PLBC405ICUADDRACK;
  input PLBC405ICUBUSY;
  input PLBC405ICUERR;
  input PLBC405ICURDBTERM;
  input PLBC405ICURDDACK;
  input [0:63] PLBC405ICURDDBUS;
  input [0:3] PLBC405ICURDWDADDR;
  input PLBC405ICUREARBITRATE;
  input PLBC405ICUWRBTERM;
  input PLBC405ICUWRDACK;
  input [0:1] PLBC405ICUSSIZE;
  input PLBC405ICUSERR;
  input PLBC405ICUSBUSYS;
  output [0:31] C405PLBDCUABUS;
  output [0:7] C405PLBDCUBE;
  output C405PLBDCURNW;
  output C405PLBDCUABORT;
  output C405PLBDCUBUSLOCK;
  output C405PLBDCUU0ATTR;
  output C405PLBDCUGUARDED;
  output C405PLBDCULOCKERR;
  output [0:1] C405PLBDCUMSIZE;
  output C405PLBDCUORDERED;
  output [0:1] C405PLBDCUPRIORITY;
  output C405PLBDCURDBURST;
  output C405PLBDCUREQUEST;
  output [0:3] C405PLBDCUSIZE;
  output [0:2] C405PLBDCUTYPE;
  output C405PLBDCUWRBURST;
  output [0:63] C405PLBDCUWRDBUS;
  output C405PLBDCUCACHEABLE;
  output C405PLBDCUWRITETHRU;
  input PLBC405DCUADDRACK;
  input PLBC405DCUBUSY;
  input PLBC405DCUERR;
  input PLBC405DCURDBTERM;
  input PLBC405DCURDDACK;
  input [0:63] PLBC405DCURDDBUS;
  input [0:3] PLBC405DCURDWDADDR;
  input PLBC405DCUREARBITRATE;
  input PLBC405DCUWRBTERM;
  input PLBC405DCUWRDACK;
  input [0:1] PLBC405DCUSSIZE;
  input PLBC405DCUSERR;
  input PLBC405DCUSBUSYS;
  input BRAMDSOCMCLK;
  input [0:31] BRAMDSOCMRDDBUS;
  input [0:7] DSARCVALUE;
  input [0:7] DSCNTLVALUE;
  output [8:29] DSOCMBRAMABUS;
  output [0:3] DSOCMBRAMBYTEWRITE;
  output DSOCMBRAMEN;
  output [0:31] DSOCMBRAMWRDBUS;
  output DSOCMBUSY;
  output DSOCMRDADDRVALID;
  output DSOCMWRADDRVALID;
  input DSOCMRWCOMPLETE;
  input BRAMISOCMCLK;
  input [0:63] BRAMISOCMRDDBUS;
  input [0:31] BRAMISOCMDCRRDDBUS;
  input [0:7] ISARCVALUE;
  input [0:7] ISCNTLVALUE;
  output ISOCMBRAMEN;
  output ISOCMBRAMEVENWRITEEN;
  output ISOCMBRAMODDWRITEEN;
  output [8:28] ISOCMBRAMRDABUS;
  output [8:28] ISOCMBRAMWRABUS;
  output [0:31] ISOCMBRAMWRDBUS;
  output ISOCMDCRBRAMEVENEN;
  output ISOCMDCRBRAMODDEN;
  output ISOCMDCRBRAMRDSELECT;
  output [8:9] DCREMACABUS;
  output DCREMACCLK;
  output [0:31] DCREMACDBUS;
  output DCREMACENABLER;
  output DCREMACREAD;
  output DCREMACWRITE;
  input EMACDCRACK;
  input [0:31] EMACDCRDBUS;
  output [0:9] EXTDCRABUS;
  output [0:31] EXTDCRDBUSOUT;
  output EXTDCRREAD;
  output EXTDCRWRITE;
  input EXTDCRACK;
  input [0:31] EXTDCRDBUSIN;
  input EICC405CRITINPUTIRQ;
  input EICC405EXTINPUTIRQ;
  output C405JTGCAPTUREDR;
  output C405JTGEXTEST;
  output C405JTGPGMOUT;
  output C405JTGSHIFTDR;
  output C405JTGTDO;
  output C405JTGTDOEN;
  output C405JTGUPDATEDR;
  input MCBJTAGEN;
  input JTGC405BNDSCANTDO;
  input JTGC405TCK;
  input JTGC405TDI;
  input JTGC405TMS;
  input JTGC405TRSTNEG;
  output C405DBGMSRWE;
  output C405DBGSTOPACK;
  output C405DBGWBCOMPLETE;
  output C405DBGWBFULL;
  output [0:29] C405DBGWBIAR;
  input DBGC405DEBUGHALT;
  input DBGC405EXTBUSHOLDACK;
  input DBGC405UNCONDDEBUGEVENT;
  output C405DBGLOADDATAONAPUDBUS;
  output C405TRCCYCLE;
  output [0:1] C405TRCEVENEXECUTIONSTATUS;
  output [0:1] C405TRCODDEXECUTIONSTATUS;
  output [0:3] C405TRCTRACESTATUS;
  output C405TRCTRIGGEREVENTOUT;
  output [0:10] C405TRCTRIGGEREVENTTYPE;
  input TRCC405TRACEDISABLE;
  input TRCC405TRIGGEREVENTIN;
endmodule

// synthesis attribute box_type of ppc405_0_wrapper is "black_box";

module uc2_reset_0_wrapper
  (
    UC2_sync_clk,
    Ext_Reset_In,
    Core_Reset_Req,
    Chip_Reset_Req,
    System_Reset_Req,
    Reset405out
  );
  input UC2_sync_clk;
  input Ext_Reset_In;
  input Core_Reset_Req;
  input Chip_Reset_Req;
  input System_Reset_Req;
  output Reset405out;
endmodule

// synthesis attribute box_type of uc2_reset_0_wrapper is "black_box";

