///////////////////////////////////////////////////////////////////////////////
//      Project:  Virtex-4 FX Ethernet MAC Wrappers
//
//         Date:  $Date: 2004/11/08 16:19:43 $
//          Tag:  $Name: build+H-39+0 $
//         File:  $RCSfile: v4_emac_v.ejava,v $
//          Rev:  $Revision: 1.1.6.2 $
//
//      Company:  Xilinx
// Contributors:  M. Low
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2004 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
//
//  emac
//
//  Author: Mary Low
//          Xilinx - Embedded Networking System Engineering Group
//
// Description:  This is the top level Verilog wrapper for the Virtex-4 FX 
//               Ethernet MAC.
//

`timescale 1 ps / 1 ps

///////////////////////////////////////////////////////////////////////////////
// The entity declaration for the top level wrapper.
///////////////////////////////////////////////////////////////////////////////
module temac
(
    // Client Receiver Interface - EMAC0
    EMAC0CLIENTRXCLIENTCLKOUT,
    CLIENTEMAC0RXCLIENTCLKIN,
    EMAC0CLIENTRXD,
    EMAC0CLIENTRXDVLD,
    EMAC0CLIENTRXDVLDMSW,
    EMAC0CLIENTRXGOODFRAME,
    EMAC0CLIENTRXBADFRAME,
    EMAC0CLIENTRXFRAMEDROP,
    EMAC0CLIENTRXDVREG6,
    EMAC0CLIENTRXSTATS,
    EMAC0CLIENTRXSTATSVLD,
    EMAC0CLIENTRXSTATSBYTEVLD,

    // Client Transmitter Interface - EMAC0
    EMAC0CLIENTTXCLIENTCLKOUT,
    CLIENTEMAC0TXCLIENTCLKIN,
    CLIENTEMAC0TXD,
    CLIENTEMAC0TXDVLD,
    CLIENTEMAC0TXDVLDMSW,
    EMAC0CLIENTTXACK,
    CLIENTEMAC0TXFIRSTBYTE,
    CLIENTEMAC0TXUNDERRUN,
    EMAC0CLIENTTXCOLLISION,
    EMAC0CLIENTTXRETRANSMIT,
    CLIENTEMAC0TXIFGDELAY,
    EMAC0CLIENTTXSTATS,
    EMAC0CLIENTTXSTATSVLD,
    EMAC0CLIENTTXSTATSBYTEVLD,

    // MAC Control Interface - EMAC0
    CLIENTEMAC0PAUSEREQ,
    CLIENTEMAC0PAUSEVAL,

    // Clock Signal - EMAC0
    GTX_CLK_0,
    EMAC0CLIENTTXGMIIMIICLKOUT,
    CLIENTEMAC0TXGMIIMIICLKIN,

    // GMII Interface - EMAC0
    GMII_TXD_0,
    GMII_TX_EN_0,
    GMII_TX_ER_0,
    GMII_TX_CLK_0,
    GMII_RXD_0,
    GMII_RX_DV_0,
    GMII_RX_ER_0,
    GMII_RX_CLK_0,
    MII_TX_CLK_0,
    GMII_COL_0,
    GMII_CRS_0,

    // MDIO Interface - EMAC0
    MDC_0,
    MDIO_IN_0,
    MDIO_OUT_0,
    MDIO_TRI_0,

    HOSTCLK,

    // Asynchronous Reset
    RESET
);

//***************************** Port Declarations ******************************

    // Client Receiver Interface - EMAC0
    output          EMAC0CLIENTRXCLIENTCLKOUT;
    input           CLIENTEMAC0RXCLIENTCLKIN;
    output   [7:0]  EMAC0CLIENTRXD;
    output          EMAC0CLIENTRXDVLD;
    output          EMAC0CLIENTRXDVLDMSW;
    output          EMAC0CLIENTRXGOODFRAME;
    output          EMAC0CLIENTRXBADFRAME;
    output          EMAC0CLIENTRXFRAMEDROP;
    output          EMAC0CLIENTRXDVREG6;
    output   [6:0]  EMAC0CLIENTRXSTATS;
    output          EMAC0CLIENTRXSTATSVLD;
    output          EMAC0CLIENTRXSTATSBYTEVLD;

    // Client Transmitter Interface - EMAC0
    output          EMAC0CLIENTTXCLIENTCLKOUT;
    input           CLIENTEMAC0TXCLIENTCLKIN;
    input    [7:0]  CLIENTEMAC0TXD;
    input           CLIENTEMAC0TXDVLD;
    input           CLIENTEMAC0TXDVLDMSW;
    output          EMAC0CLIENTTXACK;
    input           CLIENTEMAC0TXFIRSTBYTE;
    input           CLIENTEMAC0TXUNDERRUN;
    output          EMAC0CLIENTTXCOLLISION;
    output          EMAC0CLIENTTXRETRANSMIT;
    input    [7:0]  CLIENTEMAC0TXIFGDELAY;
    output          EMAC0CLIENTTXSTATS;
    output          EMAC0CLIENTTXSTATSVLD;
    output          EMAC0CLIENTTXSTATSBYTEVLD;

    // MAC Control Interface - EMAC0
    input           CLIENTEMAC0PAUSEREQ;
    input   [15:0]  CLIENTEMAC0PAUSEVAL;

    // Clock Signal - EMAC0
    input           GTX_CLK_0;
    output          EMAC0CLIENTTXGMIIMIICLKOUT;
    input           CLIENTEMAC0TXGMIIMIICLKIN;

    // GMII Interface - EMAC0
    output   [7:0]  GMII_TXD_0;
    output          GMII_TX_EN_0;
    output          GMII_TX_ER_0;
    output          GMII_TX_CLK_0;
    input    [7:0]  GMII_RXD_0;
    input           GMII_RX_DV_0;
    input           GMII_RX_ER_0;
    input           GMII_RX_CLK_0;
    input           MII_TX_CLK_0;
    input           GMII_COL_0;
    input           GMII_CRS_0;

    // MDIO Interface - EMAC0
    output          MDC_0;
    input           MDIO_IN_0;
    output          MDIO_OUT_0;
    output          MDIO_TRI_0;

    input           HOSTCLK;

    // Asynchronous Reset
    input           RESET;

//****************************** Wire Declarations *****************************
    wire    [15:0]  client_rx_data_0_i;
    wire    [15:0]  client_tx_data_0_i;

    wire    [79:0]  tieemac0configvector_i;
    wire     [4:0]  phy_config_vector_0_i;
    wire            has_mdio_0_i;
    wire     [1:0]  speed_0_i;
    wire            has_rgmii_0_i;
    wire            has_sgmii_0_i;
    wire            has_gpcs_0_i;
    wire            has_host_0_i;
    wire            tx_client_16_0_i;
    wire            rx_client_16_0_i;
    wire            addr_filter_enable_0_i;
    wire            rx_lt_check_dis_0_i;
    wire     [1:0]  flow_control_config_vector_0_i;
    wire     [6:0]  tx_config_vector_0_i;
    wire     [5:0]  rx_config_vector_0_i;
    wire    [47:0]  pause_address_0_i;

    wire    [47:0]  unicast_address_0_i;



//***************************** Main Body of Code ******************************
    // 8-bit client data on EMAC0
    assign EMAC0CLIENTRXD = client_rx_data_0_i[7:0];
    assign client_tx_data_0_i = {8'b00000000, CLIENTEMAC0TXD};


    // Unicast Address
    assign unicast_address_0_i = 48'h000000000000;

    ///////////////////////////////////////////////////////////////////////////
    // Construct the tie-off vector
    // tieemac#conficvector_i[79]: Reserved - Tie to "1"
    // tieemac#configvector_i[78:74]: phy_configuration_vector[4:0] that is used
    //     to configure the PCS/PMA either when the MDIO is not present or as
    //     initial values loaded upon reset that can be modified through the
    //     MDIO.
    // tieemac#configvector_i[73:65]: tie_off_vector[8:0] that is used to 
    //     configure the mode of the EMAC.
    // tieemac#configvector_i[64:0]  mac_configuration_vector[64:0] that is used
    //     to configure the EMAC either when the Host interface is not present
    //     or as initial values loaded upon reset that can be modified through
    //     the Host interface.
    ///////////////////////////////////////////////////////////////////////////
    // EMAC0
    assign tieemac0configvector_i = {1'b1, phy_config_vector_0_i,
                                     has_mdio_0_i,
                                     speed_0_i,
                                     has_rgmii_0_i,
                                     has_sgmii_0_i,
                                     has_gpcs_0_i,
                                     has_host_0_i,
                                     tx_client_16_0_i,
                                     rx_client_16_0_i,
                                     addr_filter_enable_0_i,
                                     rx_lt_check_dis_0_i,
                                     flow_control_config_vector_0_i,
                                     tx_config_vector_0_i,
                                     rx_config_vector_0_i,
                                     pause_address_0_i};

    assign phy_config_vector_0_i = 5'b00000;
    assign has_mdio_0_i = 1'b1;
    assign speed_0_i = 2'b01;
    assign has_rgmii_0_i = 1'b0;
    assign has_sgmii_0_i = 1'b0;
    assign has_gpcs_0_i = 1'b0;
    assign has_host_0_i = 1'b1;
    assign tx_client_16_0_i = 1'b0;
    assign rx_client_16_0_i = 1'b0;
    assign addr_filter_enable_0_i = 1'b0;
    assign rx_lt_check_dis_0_i = 1'b0;
    assign flow_control_config_vector_0_i[1] = 1'b0;
    assign flow_control_config_vector_0_i[0] = 1'b0;
    assign tx_config_vector_0_i[6] = 1'b0;
    assign tx_config_vector_0_i[5] = 1'b0;
    assign tx_config_vector_0_i[4] = 1'b0;
    assign tx_config_vector_0_i[3] = 1'b1;
    assign tx_config_vector_0_i[2] = 1'b0;
    assign tx_config_vector_0_i[1] = 1'b0;
    assign tx_config_vector_0_i[0] = 1'b0;
    assign rx_config_vector_0_i[5] = 1'b0;
    assign rx_config_vector_0_i[4] = 1'b0;
    assign rx_config_vector_0_i[3] = 1'b0;
    assign rx_config_vector_0_i[2] = 1'b1;
    assign rx_config_vector_0_i[1] = 1'b0;
    assign rx_config_vector_0_i[0] = 1'b0;
    assign pause_address_0_i = 48'h000000000000;



    ///////////////////////////////////////////////////////////////////////////
    // Instantiate V4 EMAC
    ///////////////////////////////////////////////////////////////////////////
    EMAC v4_emac
    (
        .RESET                          (RESET),

        // EMAC0
        .EMAC0CLIENTRXCLIENTCLKOUT      (EMAC0CLIENTRXCLIENTCLKOUT),
        .CLIENTEMAC0RXCLIENTCLKIN       (CLIENTEMAC0RXCLIENTCLKIN),
        .EMAC0CLIENTRXD                 (client_rx_data_0_i),
        .EMAC0CLIENTRXDVLD              (EMAC0CLIENTRXDVLD),
        .EMAC0CLIENTRXDVLDMSW           (EMAC0CLIENTRXDVLDMSW),
        .EMAC0CLIENTRXGOODFRAME         (EMAC0CLIENTRXGOODFRAME),
        .EMAC0CLIENTRXBADFRAME          (EMAC0CLIENTRXBADFRAME),
        .EMAC0CLIENTRXFRAMEDROP         (EMAC0CLIENTRXFRAMEDROP),
        .EMAC0CLIENTRXDVREG6            (EMAC0CLIENTRXDVREG6),
        .EMAC0CLIENTRXSTATS             (EMAC0CLIENTRXSTATS),
        .EMAC0CLIENTRXSTATSVLD          (EMAC0CLIENTRXSTATSVLD),
        .EMAC0CLIENTRXSTATSBYTEVLD      (EMAC0CLIENTRXSTATSBYTEVLD),

        .EMAC0CLIENTTXCLIENTCLKOUT      (EMAC0CLIENTTXCLIENTCLKOUT),
        .CLIENTEMAC0TXCLIENTCLKIN       (CLIENTEMAC0TXCLIENTCLKIN),
        .CLIENTEMAC0TXD                 (client_tx_data_0_i),
        .CLIENTEMAC0TXDVLD              (CLIENTEMAC0TXDVLD),
        .CLIENTEMAC0TXDVLDMSW           (CLIENTEMAC0TXDVLDMSW),
        .EMAC0CLIENTTXACK               (EMAC0CLIENTTXACK),
        .CLIENTEMAC0TXFIRSTBYTE         (CLIENTEMAC0TXFIRSTBYTE),
        .CLIENTEMAC0TXUNDERRUN          (CLIENTEMAC0TXUNDERRUN),
        .EMAC0CLIENTTXCOLLISION         (EMAC0CLIENTTXCOLLISION),
        .EMAC0CLIENTTXRETRANSMIT        (EMAC0CLIENTTXRETRANSMIT),
        .CLIENTEMAC0TXIFGDELAY          (CLIENTEMAC0TXIFGDELAY),
        .EMAC0CLIENTTXSTATS             (EMAC0CLIENTTXSTATS),
        .EMAC0CLIENTTXSTATSVLD          (EMAC0CLIENTTXSTATSVLD),
        .EMAC0CLIENTTXSTATSBYTEVLD      (EMAC0CLIENTTXSTATSBYTEVLD),

        .CLIENTEMAC0PAUSEREQ            (CLIENTEMAC0PAUSEREQ),
        .CLIENTEMAC0PAUSEVAL            (CLIENTEMAC0PAUSEVAL),

        .PHYEMAC0GTXCLK                 (GTX_CLK_0),
        .EMAC0CLIENTTXGMIIMIICLKOUT     (EMAC0CLIENTTXGMIIMIICLKOUT),
        .CLIENTEMAC0TXGMIIMIICLKIN      (CLIENTEMAC0TXGMIIMIICLKIN),

        .PHYEMAC0RXCLK                  (GMII_RX_CLK_0),
        .PHYEMAC0RXD                    (GMII_RXD_0),
        .PHYEMAC0RXDV                   (GMII_RX_DV_0),
        .PHYEMAC0RXER                   (GMII_RX_ER_0),
        .EMAC0PHYTXCLK                  (GMII_TX_CLK_0),
        .EMAC0PHYTXD                    (GMII_TXD_0),
        .EMAC0PHYTXEN                   (GMII_TX_EN_0),
        .EMAC0PHYTXER                   (GMII_TX_ER_0),
        .PHYEMAC0MIITXCLK               (MII_TX_CLK_0),
        .PHYEMAC0COL                    (GMII_COL_0),
        .PHYEMAC0CRS                    (GMII_CRS_0),

        .CLIENTEMAC0DCMLOCKED           (1'b1),
        .EMAC0CLIENTANINTERRUPT         (),
        .PHYEMAC0SIGNALDET              (1'b0),
        .PHYEMAC0PHYAD                  (5'b00000),
        .EMAC0PHYENCOMMAALIGN           (),
        .EMAC0PHYLOOPBACKMSB            (),
        .EMAC0PHYMGTRXRESET             (),
        .EMAC0PHYMGTTXRESET             (),
        .EMAC0PHYPOWERDOWN              (),
        .EMAC0PHYSYNCACQSTATUS          (),
        .PHYEMAC0RXCLKCORCNT            (3'b000),
        .PHYEMAC0RXBUFSTATUS            (2'b00),
        .PHYEMAC0RXBUFERR               (1'b0),
        .PHYEMAC0RXCHARISCOMMA          (1'b0),
        .PHYEMAC0RXCHARISK              (1'b0),
        .PHYEMAC0RXCHECKINGCRC          (1'b0),
        .PHYEMAC0RXCOMMADET             (1'b0),
        .PHYEMAC0RXDISPERR              (1'b0),
        .PHYEMAC0RXLOSSOFSYNC           (2'b00),
        .PHYEMAC0RXNOTINTABLE           (1'b0),
        .PHYEMAC0RXRUNDISP              (1'b0),
        .PHYEMAC0TXBUFERR               (1'b0),
        .EMAC0PHYTXCHARDISPMODE         (),
        .EMAC0PHYTXCHARDISPVAL          (),
        .EMAC0PHYTXCHARISK              (),

        .EMAC0PHYMCLKOUT                (MDC_0),
        .PHYEMAC0MCLKIN                 (1'b0),
        .PHYEMAC0MDIN                   (MDIO_IN_0),
        .EMAC0PHYMDOUT                  (MDIO_OUT_0),
        .EMAC0PHYMDTRI                  (MDIO_TRI_0),

        .TIEEMAC0CONFIGVEC              (tieemac0configvector_i),
        .TIEEMAC0UNICASTADDR            (unicast_address_0_i),

        // EMAC1
        .EMAC1CLIENTRXCLIENTCLKOUT      (),
        .CLIENTEMAC1RXCLIENTCLKIN       (1'b0),
        .EMAC1CLIENTRXD                 (),
        .EMAC1CLIENTRXDVLD              (),
        .EMAC1CLIENTRXDVLDMSW           (),
        .EMAC1CLIENTRXGOODFRAME         (),
        .EMAC1CLIENTRXBADFRAME          (),
        .EMAC1CLIENTRXFRAMEDROP         (),
        .EMAC1CLIENTRXDVREG6            (),
        .EMAC1CLIENTRXSTATS             (),
        .EMAC1CLIENTRXSTATSVLD          (),
        .EMAC1CLIENTRXSTATSBYTEVLD      (),

        .EMAC1CLIENTTXCLIENTCLKOUT      (),
        .CLIENTEMAC1TXCLIENTCLKIN       (1'b0),
        .CLIENTEMAC1TXD                 (16'h0000),
        .CLIENTEMAC1TXDVLD              (1'b0),
        .CLIENTEMAC1TXDVLDMSW           (1'b0),
        .EMAC1CLIENTTXACK               (),
        .CLIENTEMAC1TXFIRSTBYTE         (1'b0),
        .CLIENTEMAC1TXUNDERRUN          (1'b0),
        .EMAC1CLIENTTXCOLLISION         (),
        .EMAC1CLIENTTXRETRANSMIT        (),
        .CLIENTEMAC1TXIFGDELAY          (8'h00),
        .EMAC1CLIENTTXSTATS             (),
        .EMAC1CLIENTTXSTATSVLD          (),
        .EMAC1CLIENTTXSTATSBYTEVLD      (),

        .CLIENTEMAC1PAUSEREQ            (1'b0),
        .CLIENTEMAC1PAUSEVAL            (16'h0000),

        .PHYEMAC1GTXCLK                 (1'b0),
        .EMAC1CLIENTTXGMIIMIICLKOUT     (),
        .CLIENTEMAC1TXGMIIMIICLKIN      (1'b0),

        .PHYEMAC1RXCLK                  (1'b0),
        .PHYEMAC1RXD                    (8'h00),
        .PHYEMAC1RXDV                   (1'b0),
        .PHYEMAC1RXER                   (1'b0),
        .PHYEMAC1MIITXCLK               (1'b0),
        .EMAC1PHYTXCLK                  (),
        .EMAC1PHYTXD                    (),
        .EMAC1PHYTXEN                   (),
        .EMAC1PHYTXER                   (),
        .PHYEMAC1COL                    (1'b0),
        .PHYEMAC1CRS                    (1'b0),

        .CLIENTEMAC1DCMLOCKED           (1'b1),
        .EMAC1CLIENTANINTERRUPT         (),
        .PHYEMAC1SIGNALDET              (1'b0),
        .PHYEMAC1PHYAD                  (5'b00000),
        .EMAC1PHYENCOMMAALIGN           (),
        .EMAC1PHYLOOPBACKMSB            (),
        .EMAC1PHYMGTRXRESET             (),
        .EMAC1PHYMGTTXRESET             (),
        .EMAC1PHYPOWERDOWN              (),
        .EMAC1PHYSYNCACQSTATUS          (),
        .PHYEMAC1RXCLKCORCNT            (3'b000),
        .PHYEMAC1RXBUFSTATUS            (2'b00),
        .PHYEMAC1RXBUFERR               (1'b0),
        .PHYEMAC1RXCHARISCOMMA          (1'b0),
        .PHYEMAC1RXCHARISK              (1'b0),
        .PHYEMAC1RXCHECKINGCRC          (1'b0),
        .PHYEMAC1RXCOMMADET             (1'b0),
        .PHYEMAC1RXDISPERR              (1'b0),
        .PHYEMAC1RXLOSSOFSYNC           (2'b00),
        .PHYEMAC1RXNOTINTABLE           (1'b0),
        .PHYEMAC1RXRUNDISP              (1'b0),
        .PHYEMAC1TXBUFERR               (1'b0),
        .EMAC1PHYTXCHARDISPMODE         (),
        .EMAC1PHYTXCHARDISPVAL          (),
        .EMAC1PHYTXCHARISK              (),

        .EMAC1PHYMCLKOUT                (),
        .PHYEMAC1MCLKIN                 (1'b0),
        .PHYEMAC1MDIN                   (1'b0),
        .EMAC1PHYMDOUT                  (),
        .EMAC1PHYMDTRI                  (),

        .TIEEMAC1CONFIGVEC              (80'd0),
        .TIEEMAC1UNICASTADDR            (48'd0),

        // Host Interface 
        .HOSTCLK                        (HOSTCLK),
        .HOSTOPCODE                     (2'b00),
        .HOSTREQ                        (1'b0),
        .HOSTMIIMSEL                    (1'b0),
        .HOSTADDR                       (10'b0000000000),
        .HOSTWRDATA                     (32'h00000000),
        .HOSTMIIMRDY                    (),
        .HOSTRDDATA                     (),
        .HOSTEMAC1SEL                   (1'b0),

        // DCR Interface
        .DCREMACCLK                     (),
        .DCREMACABUS                    (),
        .DCREMACREAD                    (),
        .DCREMACWRITE                   (),
        .DCREMACDBUS                    (),
        .EMACDCRACK                     (),
        .EMACDCRDBUS                    (),
        .DCREMACENABLE                  (1'b1),
        .DCRHOSTDONEIR                  ()
    );

endmodule
