////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
// Module: temac_controller
////////////////////////////////////////////////////////////////////////////////
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
// 
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// Filename:     temac_controller.v
// 
// Description:    
// Module instantiates an temac and its supporting logic
//
// Design Notes:
//   
//////////////////////////////////////////////////////////////////////////////-
//////////////////////////////////////////////////////////////////////////////-
// Author:      SEG         
// History:     2005.8.10 - Initial release
// 
//
//////////////////////////////////////////////////////////////////////////////-

module temac_controller
  (
    gmii_col,
    gmii_crs,
    gmii_rx_clk ,
    gmii_rx_dv ,
    gmii_rx_er ,
    gmii_rxd ,
    mii_tx_clk ,
    gmii_tx_clk ,
    gmii_tx_en ,
    gmii_tx_er ,
    gmii_txd ,
    phy_mii_int_n,
    DSOCMBRAMWRDBUS ,
    mdc_0,
    mdio,
    sys_rst,
    phy_mii_int,
    sys_clk,
    dcm1_locked,
    gpio_in,
    rx_fifo_rst,
    tx_fifo_lock_n ,
    rx_fifo_rden ,
    tx_fifo_wren,
    tx_fifo_full ,
    clk_100mhz
    );

    input  gmii_col;
    input  gmii_crs;
    input  gmii_rx_clk;
    input  gmii_rx_dv;
    input  gmii_rx_er;
    input  [0:7] gmii_rxd;
    input  mii_tx_clk;
    output gmii_tx_clk;
    output gmii_tx_en;
    output gmii_tx_er;
    output [0:7] gmii_txd; 
    input  phy_mii_int_n;
    input  [0:31] DSOCMBRAMWRDBUS;
    output mdc_0;
    inout  mdio;
    input  sys_rst;
    output phy_mii_int;
    input  sys_clk;
    output dcm1_locked;
    output [0:31] gpio_in ;
    input  rx_fifo_rst;
    input  tx_fifo_lock_n;
    input  rx_fifo_rden;
    input  tx_fifo_wren;
    output tx_fifo_full;
    input  clk_100mhz;


// DCM Signals
    wire gtx_clk_bufg_in;
    wire dcm1_clk0;

// EMAC signal
    wire reset_i;
    wire gtx_clk_bufg_0_i;
    wire rx_client_clk_out_0_i;
    wire rx_client_clk_in_0_i;
    wire tx_client_clk_out_0_i;
    wire tx_client_clk_in_0_i;
    wire tx_gmii_mii_clk_out_0_i;
    wire tx_gmii_mii_clk_in_0_i;
    wire gmii_tx_clk_0_i;
    wire gmii_tx_en_0_i;
    wire gmii_tx_er_0_i;
    wire [7:0] gmii_txd_0_i;
    wire mii_tx_clk_0_i;
    wire gmii_col_0_i;
    wire gmii_crs_0_i;
    wire gmii_rx_clk_ibufg_0_i;
    wire gmii_rx_clk_0_i;
    wire gmii_rx_dv_0_i;
    wire gmii_rx_dv_delay_0_i;
    wire gmii_rx_er_0_i;
    wire gmii_rx_er_delay_0_i;
    wire [7:0]gmii_rxd_0_i;
    wire [7:0]gmii_rxd_delay_0_i;
    reg  gmii_tx_en_0_r;
    reg  gmii_tx_er_0_r;
    reg  [7:0]gmii_txd_0_r;
    reg  gmii_rx_dv_0_r;
    reg  gmii_rx_er_0_r;
    reg  [7:0]gmii_rxd_0_r;


    // MDIO wires
    wire MDIO_IN_0;
    reg  MDIO_OUT_0;
    reg  MDIO_TRI_0;
    reg  mdc_0_obuf;
    wire emac_mdc;
    reg  emac_mdio_in;
    wire emac_mdio_out;
    wire emac_mdio_tri;

    // phy interrupt
    wire [0:0]phy_mii_int_n_IBUF;


    // EMAC Receive interface to rx_fifo
    wire rx_valid;
    wire rx_good_frame ;
    wire rx_bad_frame ;
    wire [0:7]rx_data ;
    wire rx_clk ;
    wire [7:0]rx_fifo_DO;
    wire rx_fifo_wren ;
    wire rx_fifo_full ;
    wire rx_valid_fifo ;
    wire rx_fifo_rderr ;


    // EMAC Transmit interface to tx_fifo
    wire [7:0]tx_data ;
    wire tx_valid ;
    wire tx_ack ;
    reg  tx_ack_delay ;
    wire n_tx_valid; 
    wire tx_fifo_empty ;
    wire tx_fifo_rden ;
    wire [8:0] tx_fifo_di ;
    wire tx_fifo_full_wire;
    wire tx_clk;
    reg  tx_fifo_rden_start;
    wire [31:0]tx_data_bus;
    wire [3:0]tx_valid_bus;
    wire [31:0]rx_fifo_DO_bus;
    wire [3:0]rx_valid_fifo_bus;


    // tx fifo supporting logic
    assign tx_fifo_full    = tx_fifo_full_wire;
    assign tx_fifo_di[8:0] = DSOCMBRAMWRDBUS[23:31];
    assign tx_fifo_rden    = (tx_fifo_rden_start  || tx_ack_delay || tx_ack); 

    always @ (posedge tx_clk or posedge sys_rst)
    begin
      if (sys_rst)
        tx_fifo_rden_start <= 1'b0;
      else
        tx_fifo_rden_start <=((~tx_fifo_empty)&& (tx_fifo_lock_n || tx_fifo_full_wire) && (n_tx_valid)); 
    end 

    always @ (posedge tx_clk)
    begin
      if ((n_tx_valid) || sys_rst) 
        tx_ack_delay <= 1'b0;
      else 
        begin 
          if (tx_ack)  
            tx_ack_delay <=1'b1;
        end
    end 

// rx FIFO supporting logic
    assign rx_fifo_wren = (rx_valid || rx_good_frame || rx_bad_frame ) && (~ rx_fifo_rst) ;

// connection to  ppc from rx fifo
    assign gpio_in[24:31]  = rx_fifo_DO;
    assign gpio_in[8]      = rx_fifo_rderr; 
    assign gpio_in[11]     = rx_valid_fifo;
    assign gpio_in[13]     = rx_fifo_full;
    assign gpio_in[0:7]    = 8'b00000000;
    assign gpio_in[9:10]   = 2'b00;
    assign gpio_in[12]     = 1'b0;
    assign gpio_in[14:23]  = 10'b0000000000;


//////////////////////////////////////////////////////////////////////////////////-
//ETHERNET SIGNALS, BUFFERS, FLIPFLIPS, AND EMAC
//////////////////////////////////////////////////////////////////////////////////-

    assign phy_mii_int = ~ phy_mii_int_n_IBUF[0];
    IBUF phy_int_ibuf(.I(phy_mii_int_n), .O(phy_mii_int_n_IBUF[0]));
    //*************************** Main Body of Code ****************************
    assign reset_i = sys_rst;
    IOBUF mdio_ibuf (.I(MDIO_OUT_0), .IO(mdio), .O(MDIO_IN_0), .T(MDIO_TRI_0));
    OBUF mdc_obuf (.I(mdc_0_obuf), .O(mdc_0));
    always @ (posedge sys_clk)
    begin
          mdc_0_obuf<= emac_mdc ;
          MDIO_OUT_0<=emac_mdio_out;
          MDIO_TRI_0<=emac_mdio_tri;
    end 
    always @ (negedge mdc_0_obuf)
    begin
          emac_mdio_in<= MDIO_IN_0 ;
    end 


    //********** EMAC0 **********
    ////////////////////////////////////////////////////////////////////////////
    // GTX_CLK Clock Management - 125 MHz clock frequency from DCM1
    ////////////////////////////////////////////////////////////////////////////
    BUFG gtx_clk_bufg
    (
        .I(gtx_clk_bufg_in),
        .O(gtx_clk_bufg_0_i)
    );
    BUFG tx_gmii_mii_clk_0_bufg
    (
        .I(tx_gmii_mii_clk_out_0_i),
        .O(tx_gmii_mii_clk_in_0_i)
    );
    ////////////////////////////////////////////////////////////////////////////
    // GMII Transmitter Clock Management
    ////////////////////////////////////////////////////////////////////////////
    OBUF gmii_tx_clk_0_obuf (.I(gmii_tx_clk_0_i), .O(gmii_tx_clk));
    ////////////////////////////////////////////////////////////////////////////
    // GMII Transmitter Logic : Drive TX signals through IOBs onto GMII
    // interface
    ////////////////////////////////////////////////////////////////////////////
    always @(posedge tx_gmii_mii_clk_in_0_i or posedge reset_i)
    begin
        if (reset_i)
        begin
            gmii_tx_en_0_r <= 1'b0;
            gmii_tx_er_0_r <= 1'b0;
            gmii_txd_0_r   <= 8'h00;
        end
        else
        begin
            gmii_tx_en_0_r <= gmii_tx_en_0_i;
            gmii_tx_er_0_r <= gmii_tx_er_0_i;
            gmii_txd_0_r   <= gmii_txd_0_i;
        end
    end

    // Drive GMII TX signals through Output Buffers and onto PADS
    OBUF gmii_tx_en_0_obuf (.I(gmii_tx_en_0_r), .O(gmii_tx_en));
    OBUF gmii_tx_er_0_obuf (.I(gmii_tx_er_0_r), .O(gmii_tx_er));

    OBUF gmii_txd_0_obuf0 (.I(gmii_txd_0_r[0]), .O(gmii_txd[0]));
    OBUF gmii_txd_0_obuf1 (.I(gmii_txd_0_r[1]), .O(gmii_txd[1]));
    OBUF gmii_txd_0_obuf2 (.I(gmii_txd_0_r[2]), .O(gmii_txd[2]));
    OBUF gmii_txd_0_obuf3 (.I(gmii_txd_0_r[3]), .O(gmii_txd[3]));
    OBUF gmii_txd_0_obuf4 (.I(gmii_txd_0_r[4]), .O(gmii_txd[4]));
    OBUF gmii_txd_0_obuf5 (.I(gmii_txd_0_r[5]), .O(gmii_txd[5]));
    OBUF gmii_txd_0_obuf6 (.I(gmii_txd_0_r[6]), .O(gmii_txd[6]));
    OBUF gmii_txd_0_obuf7 (.I(gmii_txd_0_r[7]), .O(gmii_txd[7]));


    ////////////////////////////////////////////////////////////////////////////
    // GMII Receiver Clock Management
    ////////////////////////////////////////////////////////////////////////////
    IBUFG gmii_rx_clk_0_ibufg (.I(gmii_rx_clk), .O(gmii_rx_clk_ibufg_0_i));
    BUFG  gmii_rx_clk_0_bufg  (.I(gmii_rx_clk_ibufg_0_i), .O(gmii_rx_clk_0_i)); 
    ////////////////////////////////////////////////////////////////////////////
    // GMII Receiver Logic : Receive RX signals through IOBs from GMII
    // interface
    ////////////////////////////////////////////////////////////////////////////

    IBUF gmii_rx_dv_0_ibuf (.I(gmii_rx_dv), .O(gmii_rx_dv_0_i));

    IDELAY gmii_rx_dv_0_delay
    (
        .I(gmii_rx_dv_0_i),
        .O(gmii_rx_dv_delay_0_i),
        .C(1'b0),
        .CE(1'b0),
        .INC(1'b0),
        .RST(1'b0)
    ); 

    IBUF gmii_rx_er_0_ibuf (.I(gmii_rx_er), .O(gmii_rx_er_0_i));

    IDELAY gmii_rx_er_0_delay
    (
        .I(gmii_rx_er_0_i),
        .O(gmii_rx_er_delay_0_i),
        .C(1'b0),
        .CE(1'b0),
        .INC(1'b0),
        .RST(1'b0)
    );

    IBUF gmii_rxd_0_ibuf0 (.I(gmii_rxd[0]), .O(gmii_rxd_0_i[0]));
    IBUF gmii_rxd_0_ibuf1 (.I(gmii_rxd[1]), .O(gmii_rxd_0_i[1]));
    IBUF gmii_rxd_0_ibuf2 (.I(gmii_rxd[2]), .O(gmii_rxd_0_i[2]));
    IBUF gmii_rxd_0_ibuf3 (.I(gmii_rxd[3]), .O(gmii_rxd_0_i[3]));
    IBUF gmii_rxd_0_ibuf4 (.I(gmii_rxd[4]), .O(gmii_rxd_0_i[4]));
    IBUF gmii_rxd_0_ibuf5 (.I(gmii_rxd[5]), .O(gmii_rxd_0_i[5]));
    IBUF gmii_rxd_0_ibuf6 (.I(gmii_rxd[6]), .O(gmii_rxd_0_i[6]));
    IBUF gmii_rxd_0_ibuf7 (.I(gmii_rxd[7]), .O(gmii_rxd_0_i[7]));
    IDELAY gmii_rxd_0_delay0(.I(gmii_rxd_0_i[0]), .O(gmii_rxd_delay_0_i[0]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));
    IDELAY gmii_rxd_0_delay1(.I(gmii_rxd_0_i[1]), .O(gmii_rxd_delay_0_i[1]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));
    IDELAY gmii_rxd_0_delay2(.I(gmii_rxd_0_i[2]), .O(gmii_rxd_delay_0_i[2]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));
    IDELAY gmii_rxd_0_delay3(.I(gmii_rxd_0_i[3]), .O(gmii_rxd_delay_0_i[3]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));
    IDELAY gmii_rxd_0_delay4(.I(gmii_rxd_0_i[4]), .O(gmii_rxd_delay_0_i[4]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));
    IDELAY gmii_rxd_0_delay5(.I(gmii_rxd_0_i[5]), .O(gmii_rxd_delay_0_i[5]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));
    IDELAY gmii_rxd_0_delay6(.I(gmii_rxd_0_i[6]), .O(gmii_rxd_delay_0_i[6]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));
    IDELAY gmii_rxd_0_delay7(.I(gmii_rxd_0_i[7]), .O(gmii_rxd_delay_0_i[7]), .C(1'b0), .CE(1'b0), .INC(1'b0), .RST(1'b0));



    // Infer IOB Input flip-flops
    always @(posedge gmii_rx_clk_0_i or posedge reset_i)
    begin
        if (reset_i)
        begin
            gmii_rx_dv_0_r <= 1'b0;
            gmii_rx_er_0_r <= 1'b0;
            gmii_rxd_0_r   <= 8'h00;
        end
        else
        begin
            gmii_rx_dv_0_r <= gmii_rx_dv_delay_0_i;
            gmii_rx_er_0_r <= gmii_rx_er_delay_0_i;
            gmii_rxd_0_r   <= gmii_rxd_delay_0_i;
        end
    end

    // MII_TX_CLK
    IBUFG mii_tx_clk_0_ibufg (.I(mii_tx_clk), .O(mii_tx_clk_0_i)); 
    // Half Duplex signals for tri-speed mode
    IBUF gmii_col_0_obuf (.I(gmii_col), .O(gmii_col_0_i));
    IBUF gmii_crs_0_obuf (.I(gmii_crs), .O(gmii_crs_0_i));


    ////////////////////////////////////////////////////////////////////////////
    // Receive & Transmit Client Clock Management when configured in 8-bit mode
    ////////////////////////////////////////////////////////////////////////////
    BUFG rx_client_clk_0_bufg(.I(rx_client_clk_out_0_i), .O(rx_client_clk_in_0_i));
    BUFG tx_client_clk_0_bufg(.I(tx_client_clk_out_0_i), .O(tx_client_clk_in_0_i));
    assign rx_clk = rx_client_clk_in_0_i;
    assign tx_clk = tx_client_clk_in_0_i;
    ////////////////////////////////////////////////////////////////////////////
    // Instantiate V4 EMAC
    ////////////////////////////////////////////////////////////////////////////
    temac v4_emac_top
    (
        // Client Receiver Interface - EMAC0
        .EMAC0CLIENTRXCLIENTCLKOUT      (rx_client_clk_out_0_i),
        .CLIENTEMAC0RXCLIENTCLKIN       (rx_client_clk_in_0_i),
        .EMAC0CLIENTRXD                 (rx_data),
        .EMAC0CLIENTRXDVLD              (rx_valid),
        .EMAC0CLIENTRXDVLDMSW           (),
        .EMAC0CLIENTRXGOODFRAME         (rx_good_frame),
        .EMAC0CLIENTRXBADFRAME          (rx_bad_frame),
        .EMAC0CLIENTRXFRAMEDROP         (),
        .EMAC0CLIENTRXDVREG6            (),
        .EMAC0CLIENTRXSTATS             (),
        .EMAC0CLIENTRXSTATSVLD          (),
        .EMAC0CLIENTRXSTATSBYTEVLD      (),

        // Client Transmitter Interface - EMAC0
        .EMAC0CLIENTTXCLIENTCLKOUT      (tx_client_clk_out_0_i),
        .CLIENTEMAC0TXCLIENTCLKIN       (tx_client_clk_in_0_i),
        .CLIENTEMAC0TXD                 (tx_data),
        .CLIENTEMAC0TXDVLD              (tx_valid),
        .CLIENTEMAC0TXDVLDMSW           (1'b0),
        .EMAC0CLIENTTXACK               (tx_ack),
        .CLIENTEMAC0TXFIRSTBYTE         (1'b0),
        .CLIENTEMAC0TXUNDERRUN          (1'b0),
        .EMAC0CLIENTTXCOLLISION         (),
        .EMAC0CLIENTTXRETRANSMIT        (),
        .CLIENTEMAC0TXIFGDELAY          (8'b00000000),
        .EMAC0CLIENTTXSTATS             (),
        .EMAC0CLIENTTXSTATSVLD          (),
        .EMAC0CLIENTTXSTATSBYTEVLD      (),

        // MAC Control Interface - EMAC0
        .CLIENTEMAC0PAUSEREQ            (1'b0),
        .CLIENTEMAC0PAUSEVAL            (16'b0000000000000000),

        // Clock Signal - EMAC0
        .GTX_CLK_0                      (gtx_clk_bufg_0_i),
        .EMAC0CLIENTTXGMIIMIICLKOUT     (tx_gmii_mii_clk_out_0_i),
        .CLIENTEMAC0TXGMIIMIICLKIN      (tx_gmii_mii_clk_in_0_i),

        // GMII Interface - EMAC0
        .GMII_TXD_0                     (gmii_txd_0_i),
        .GMII_TX_EN_0                   (gmii_tx_en_0_i),
        .GMII_TX_ER_0                   (gmii_tx_er_0_i),
        .GMII_TX_CLK_0                  (gmii_tx_clk_0_i),
        .GMII_RXD_0                     (gmii_rxd_0_r),
        .GMII_RX_DV_0                   (gmii_rx_dv_0_r),
        .GMII_RX_ER_0                   (gmii_rx_er_0_r),
        .GMII_RX_CLK_0                  (gmii_rx_clk_0_i),
        .MII_TX_CLK_0                   (mii_tx_clk_0_i),
        .GMII_COL_0                     (gmii_col_0_i),
        .GMII_CRS_0                     (gmii_crs_0_i),

        // MDIO Interface - EMAC0
        .MDC_0                          (emac_mdc),
        .MDIO_IN_0                      (emac_mdio_in),
        .MDIO_OUT_0                     (emac_mdio_out),
        .MDIO_TRI_0                     (emac_mdio_tri),

        .HOSTCLK                        (sys_clk),

        // Asynchronous Reset
        .RESET                          (reset_i)
        );




// DCM providing 125MHz gtx_clk for temac

   DCM DCM1(
     .CLK0     (dcm1_clk0),    
     .CLK180   (),
     .CLK270   (),
     .CLK2X    (),
     .CLK2X180 (),
     .CLK90    (),
     .CLKDV    (),
     .CLKFB    (dcm1_clk0),
     .CLKFX    (gtx_clk_bufg_in),    
     .CLKFX180 (),
     .CLKIN    (clk_100mhz),     
     .DSSEN    (1'b0),
     .LOCKED   (dcm1_locked), 
     .PSCLK    (1'b0),        
     .PSDONE   (), 
     .PSEN     (1'b0),        
     .PSINCDEC (1'b0),        
     .RST      (),   
     .STATUS   ()
     );
    defparam DCM1.CLKIN_PERIOD       = 10;
    defparam DCM1.DFS_FREQUENCY_MODE = "LOW";
    defparam DCM1.CLKFX_DIVIDE       = 4;
    defparam DCM1.CLKFX_MULTIPLY     = 5;
    defparam DCM1.CLK_FEEDBACK       = "1X"; 

//////////////////////////////////////////////////////////////////
// FIFO16: Virtex-4 2k deep x 9 wide BlockRAM Asynchrnous FIFO
//////////////////////////////////////////////////////////////////


    FIFO16 tx_fifo (
      .ALMOSTEMPTY    (),
      .ALMOSTFULL     (),
      .DO             (tx_data_bus), 
      .DOP            (tx_valid_bus),
      .EMPTY          (tx_fifo_empty),
      .FULL           (tx_fifo_full_wire), 
      .RDCOUNT        (),
      .RDERR          (),
      .WRCOUNT        (),
      .WRERR          (),
      .DI             ({24'b000000000000000000000000, tx_fifo_di[7:0]}), 
      .DIP            ({3'b000, ~tx_fifo_di[8]}), 
      .RDCLK          (tx_clk),
      .RDEN           (tx_fifo_rden),
      .RST            (sys_rst), 
      .WRCLK          (sys_clk),  
      .WREN           (tx_fifo_wren)
    );
    assign tx_data    = tx_data_bus[7:0] ;
    assign tx_valid   = tx_valid_bus[0];
    assign n_tx_valid = ~ tx_valid;
    defparam tx_fifo.DATA_WIDTH              = 9;
    defparam tx_fifo.FIRST_WORD_FALL_THROUGH = "FALSE";
    defparam tx_fifo.ALMOST_FULL_OFFSET      = 12'b000000010000;
    defparam tx_fifo.ALMOST_EMPTY_OFFSET     = 12'b000000010000;

    FIFO16 rx_fifo(
      .ALMOSTEMPTY    (), 
      .ALMOSTFULL     (), 
      .DO             (rx_fifo_DO_bus), 
      .DOP            (rx_valid_fifo_bus), 
      .EMPTY          (),
      .FULL           (rx_fifo_full), 
      .RDCOUNT        (),
      .RDERR          (rx_fifo_rderr), 
      .WRCOUNT        (),
      .WRERR          (),
      .DI             ({24'b000000000000000000000000,rx_data}),  
      .DIP            ({3'b000, rx_valid}), 
      .RDCLK          (sys_clk), 
      .RDEN           (rx_fifo_rden),
      .RST            (rx_fifo_rst),
      .WRCLK          (rx_clk),
      .WREN           (rx_fifo_wren)
    );
    defparam rx_fifo.DATA_WIDTH              = 9;
    defparam rx_fifo.FIRST_WORD_FALL_THROUGH = "FALSE";
    defparam rx_fifo.ALMOST_FULL_OFFSET      = 12'b000000010000;
    defparam rx_fifo.ALMOST_EMPTY_OFFSET     = 12'b000000010000;
    assign rx_fifo_DO                        = rx_fifo_DO_bus[7:0];
    assign rx_valid_fifo                     = rx_valid_fifo_bus[0];
endmodule
