--------------------------------------------------------------------------------
--
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--     
--     (c) Copyright 2005 Xilinx, Inc.
--     All rights reserved.
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Filename:     temac_controller.vhd
-- 
-- Description:    
-- Module instantiates an UltraController2 for Demonstration
--
-- Design Notes:
--   
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Author:      SEG         
-- History:     2005.8.10 - Initial release
-- 
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

entity temac_controller is
  port (
    gmii_col          : in std_logic;
    gmii_crs          : in std_logic;
    gmii_rx_clk       : in std_logic;
    gmii_rx_dv        : in std_logic;
    gmii_rx_er        : in std_logic;
    gmii_rxd          : in std_logic_vector (0 to 7);
    mii_tx_clk        : in std_logic ;
    gmii_tx_clk       : out std_logic;
    gmii_tx_en        : out std_logic;
    gmii_tx_er        : out std_logic;
    gmii_txd          : out std_logic_vector(0 to 7); 
    phy_mii_int_n     : in  std_logic;
    DSOCMBRAMWRDBUS   : in std_logic_vector (0 to 31);
    MDC_0             : out std_logic;
    mdio              : inout std_logic;
    sys_rst           : in std_logic;
    phy_mii_int       : out std_logic;
    sys_clk           : in std_logic;
    dcm1_locked       : out std_logic;
    gpio_in           : out std_logic_vector (0 to 31);
    rx_fifo_rst       : in std_logic;
    tx_fifo_lock_n    : in std_logic;
    rx_fifo_rden      : in std_logic;
    tx_fifo_wren      : in std_logic;
    tx_fifo_full      : out std_logic;
    clk_100mhz        : in std_logic
    );
end temac_controller;


architecture STRUCTURE of temac_controller is
   attribute box_type : STRING;

component FIFO16
  generic(
    ALMOST_FULL_OFFSET        : bit_vector := X"010";
    ALMOST_EMPTY_OFFSET       : bit_vector := X"010";
    DATA_WIDTH                : integer := 9;
    FIRST_WORD_FALL_THROUGH   : boolean := FALSE
  );
  port(
    ALMOSTEMPTY   : out std_ulogic;
    ALMOSTFULL    : out std_ulogic;
    DO            : out std_logic_vector (31 downto 0);
    DOP           : out std_logic_vector (3 downto 0);
    EMPTY         : out std_ulogic;
    FULL          : out std_ulogic;
    RDCOUNT       : out std_logic_vector (11 downto 0);
    RDERR         : out std_ulogic;
    WRCOUNT       : out std_logic_vector (11 downto 0);
    WRERR         : out std_ulogic;
    DI            : in std_logic_vector (31 downto 0);
    DIP           : in std_logic_vector (3 downto 0);
    RDCLK         : in std_ulogic;
    RDEN          : in std_ulogic;
    RST           : in std_ulogic;
    WRCLK         : in std_ulogic;
    WREN          : in std_ulogic
  );
end component;


component temac is
    port(
        -- Client Receiver Interface - EMAC0
        EMAC0CLIENTRXCLIENTCLKOUT       : out std_logic;
        CLIENTEMAC0RXCLIENTCLKIN        : in  std_logic;
        EMAC0CLIENTRXD                  : out std_logic_vector(7 downto 0);
        EMAC0CLIENTRXDVLD               : out std_logic;
        EMAC0CLIENTRXDVLDMSW            : out std_logic;
        EMAC0CLIENTRXGOODFRAME          : out std_logic;
        EMAC0CLIENTRXBADFRAME           : out std_logic;
        EMAC0CLIENTRXFRAMEDROP          : out std_logic;
        EMAC0CLIENTRXDVREG6             : out std_logic;
        EMAC0CLIENTRXSTATS              : out std_logic_vector(6 downto 0);
        EMAC0CLIENTRXSTATSVLD           : out std_logic;
        EMAC0CLIENTRXSTATSBYTEVLD       : out std_logic;

        -- Client Transmitter Interface - EMAC0
        EMAC0CLIENTTXCLIENTCLKOUT       : out std_logic;
        CLIENTEMAC0TXCLIENTCLKIN        : in  std_logic;
        CLIENTEMAC0TXD                  : in  std_logic_vector(7 downto 0);
        CLIENTEMAC0TXDVLD               : in  std_logic;
        CLIENTEMAC0TXDVLDMSW            : in  std_logic;
        EMAC0CLIENTTXACK                : out std_logic;
        CLIENTEMAC0TXFIRSTBYTE          : in  std_logic;
        CLIENTEMAC0TXUNDERRUN           : in  std_logic;
        EMAC0CLIENTTXCOLLISION          : out std_logic;
        EMAC0CLIENTTXRETRANSMIT         : out std_logic;
        CLIENTEMAC0TXIFGDELAY           : in  std_logic_vector(7 downto 0);
        EMAC0CLIENTTXSTATS              : out std_logic;
        EMAC0CLIENTTXSTATSVLD           : out std_logic;
        EMAC0CLIENTTXSTATSBYTEVLD       : out std_logic;

        -- MAC Control Interface - EMAC0
        CLIENTEMAC0PAUSEREQ             : in  std_logic;
        CLIENTEMAC0PAUSEVAL             : in  std_logic_vector(15 downto 0);

        -- Clock Signal - EMAC0
        GTX_CLK_0                       : in  std_logic;
        EMAC0CLIENTTXGMIIMIICLKOUT      : out std_logic;
        CLIENTEMAC0TXGMIIMIICLKIN       : in  std_logic ;

        -- GMII Interface - EMAC0
        GMII_TXD_0                      : out std_logic_vector(7 downto 0);
        GMII_TX_EN_0                    : out std_logic;
        GMII_TX_ER_0                    : out std_logic;
        GMII_TX_CLK_0                   : out std_logic;
        GMII_RXD_0                      : in  std_logic_vector(7 downto 0);
        GMII_RX_DV_0                    : in  std_logic;
        GMII_RX_ER_0                    : in  std_logic;
        GMII_RX_CLK_0                   : in  std_logic;
    
        MII_TX_CLK_0                    : in  std_logic;
        GMII_COL_0                      : in  std_logic;
        GMII_CRS_0                      : in  std_logic;


        -- MDIO Interface - EMAC0
        MDC_0                           : out std_logic;
        MDIO_IN_0                       : in  std_logic;
        MDIO_OUT_0                      : out std_logic;
        MDIO_TRI_0                      : out std_logic;
  
        HOSTCLK                         : in  std_logic;
        -- Asynchronous Reset
        RESET                           : in  std_logic
        );
end component;



component DCM is
generic (
        CLKIN_PERIOD         : integer := 10;
        DFS_FREQUENCY_MODE   : string := "LOW";
        CLKFX_DIVIDE         : integer := 1;
        CLKFX_MULTIPLY       : integer := 1;
        CLK_FEEDBACK         : string :="NONE" 
);
port
    (
     CLK0     : out std_logic;      
     CLK180   : out std_logic;  
     CLK270   : out std_logic;  
     CLK2X    : out std_logic;  
     CLK2X180 : out std_logic;  
     CLK90    : out std_logic;  
     CLKDV    : out std_logic;    
     CLKFB    : in  std_logic;    
     CLKFX    : out std_logic;
     CLKFX180 : out std_logic;      
     CLKIN    : in  std_logic;     
     DSSEN    : in  std_logic;       
     LOCKED   : out std_logic;
     PSCLK    : in  std_logic;       
     PSDONE   : out std_logic;    
     PSEN     : in  std_logic;    
     PSINCDEC : in  std_logic;     
     RST      : in  std_logic;  
     STATUS   : out std_logic_vector (7 downto 0)   
     );
  end component;
attribute box_type of DCM: component is "black_box";

-- DCM Signals
   signal gtx_clk_bufg_in  :std_logic;
   signal dcm1_clk0        :std_logic;

-- EMAC signal
    signal gnd_i                          : std_logic;
    signal reset_i                        : std_logic;
    signal gtx_clk_bufg_0_i               : std_logic;
    signal rx_client_clk_out_0_i          : std_logic;
    signal rx_client_clk_in_0_i           : std_logic;
    signal tx_client_clk_out_0_i          : std_logic;
    signal tx_client_clk_in_0_i           : std_logic;
    signal tx_gmii_mii_clk_out_0_i        : std_logic;
    signal tx_gmii_mii_clk_in_0_i         : std_logic;
    signal gmii_tx_clk_0_i                : std_logic;
    signal gmii_tx_en_0_i                 : std_logic;
    signal gmii_tx_er_0_i                 : std_logic;
    signal gmii_txd_0_i                   : std_logic_vector(7 downto 0);
    signal gmii_tx_en_0_r                 : std_logic;
    signal gmii_tx_er_0_r                 : std_logic;
    signal gmii_txd_0_r                   : std_logic_vector(7 downto 0);
    signal mii_tx_clk_0_i                 : std_logic;
    signal gmii_col_0_i                   : std_logic;
    signal gmii_crs_0_i                   : std_logic;
    signal gmii_rx_clk_ibufg_0_i          : std_logic;
    signal gmii_rx_clk_0_i                : std_logic;
    signal gmii_rx_dv_0_i                 : std_logic;
    signal gmii_rx_dv_delay_0_i           : std_logic;
    signal gmii_rx_er_0_i                 : std_logic;
    signal gmii_rx_er_delay_0_i           : std_logic;
    signal gmii_rxd_0_i                   : std_logic_vector(7 downto 0);
    signal gmii_rxd_delay_0_i             : std_logic_vector(7 downto 0);
    signal gmii_rx_dv_0_r                 : std_logic;
    signal gmii_rx_er_0_r                 : std_logic;
    signal gmii_rxd_0_r                   : std_logic_vector(7 downto 0);

    -- MDIO signals
    signal MDIO_IN_0                      :  std_logic;
    signal MDIO_OUT_0                     :  std_logic;
    signal MDIO_TRI_0                     :  std_logic;
    signal mdc_0_obuf                     :  std_logic;
    signal emac_mdc                       :  std_logic;
    signal emac_mdio_in                   :  std_logic;
    signal emac_mdio_out                  :  std_logic;
    signal emac_mdio_tri                  :  std_logic;
-- phy interrupt
    signal phy_mii_int_n_IBUF             :  std_logic_vector (0 to 0);


-- EMAC Receive interface to rx_fifo
    signal rx_valid       : std_logic;
    signal rx_good_frame  : std_logic;
    signal rx_bad_frame   : std_logic;
    signal rx_data        : std_logic_vector (0 to 7);
    signal rx_clk         : std_logic;
    signal rx_fifo_DO     : std_logic_vector (7 downto 0);
    signal rx_fifo_wren   : std_logic;
    signal rx_fifo_full   : std_logic;
    signal rx_valid_fifo  : std_logic;
    signal rx_fifo_rderr  : std_logic;


-- EMAC Transmit interface to rx_fifo
    signal tx_data            : std_logic_vector (7 downto 0);
    signal tx_valid           : std_logic;
    signal n_tx_valid         : std_logic;
    signal tx_ack             : std_logic;
    signal tx_ack_delay       : std_logic;
    signal tx_fifo_empty      : std_logic;
    signal tx_fifo_rden       : std_logic;
    signal tx_fifo_di         : std_logic_vector (8 downto 0);
    signal tx_fifo_full_wire  : std_logic;
    signal tx_clk             : std_logic;
    signal tx_fifo_rden_start : std_logic;
    signal n_tx_fifo_di8      : std_logic;

    attribute IOBDELAY_TYPE  : string;
    attribute IOBDELAY_VALUE : integer;
begin

-- tx fifo supporting logic
    tx_fifo_full <= tx_fifo_full_wire;
    tx_fifo_di   <= DSOCMBRAMWRDBUS(23 to 31);
    tx_fifo_rden <= (tx_fifo_rden_start or tx_ack_delay or tx_ack); 

    rden_start :process (tx_clk, sys_rst)
    begin
        if sys_rst = '1'  then
            tx_fifo_rden_start <= '0';
        elsif tx_clk'event and tx_clk = '1' then
            tx_fifo_rden_start <=((not tx_fifo_empty)and (tx_fifo_lock_n or tx_fifo_full_wire) and (n_tx_valid));
        end if;
    end process  rden_start;

    ack_delay : process (tx_clk)
    begin
        if tx_clk'event and tx_clk = '1'  then
            if (tx_valid='0' or sys_rst='1') then 
                tx_ack_delay <= '0';
            elsif tx_ack ='1' then
                tx_ack_delay <='1';
            end if;
        end if;
    end process ack_delay;

-- rx FIFO supporting logic
    rx_fifo_wren <= (rx_valid or rx_good_frame or rx_bad_frame) and (not rx_fifo_rst);
-- connection to  ppc from rx fifo
    gpio_in(24 to 31)  <= rx_fifo_DO;
    gpio_in(8)         <= rx_fifo_rderr; 
    gpio_in(11)        <= rx_valid_fifo;
    gpio_in(13)        <= rx_fifo_full;
    gpio_in(0 to 7)    <= "00000000";
    gpio_in(9 to 10)   <= "00";
    gpio_in(12)        <= '0';
    gpio_in(14 to 23)  <= "0000000000";

-----------------------------------------------------------------------------------
--ETHERNET SIGNALS, BUFFERS, FLIPFLIPS, AND EMAC
-----------------------------------------------------------------------------------

    phy_mii_int<= not phy_mii_int_n_IBUF(0);
     ibuf_29 : IBUF port map(
     I=> phy_mii_int_n,
     O=> phy_mii_int_n_IBUF(0)
     );
 
    obuf55 : OBUF port map(
     I=>mdc_0_obuf,
     O=>mdc_0
    );
    --*************************** Main Body of Code ****************************
    gnd_i     <= '0';

    reset_i <= sys_rst;
    iobuf_56:IOBUF port map(
                  I=>MDIO_OUT_0,
                  IO=>mdio,
                  O=>MDIO_IN_0,
                  T=>MDIO_TRI_0
    );

    mdio_reg1 : process (sys_clk)
    begin
        if sys_clk'event and sys_clk = '1' then
          mdc_0_obuf<= emac_mdc ;
          MDIO_OUT_0<=emac_mdio_out;
          MDIO_TRI_0<=emac_mdio_tri;
        end if;
    end process mdio_reg1;
    mdio_reg2 : process (mdc_0_obuf)
    begin
        if mdc_0_obuf'event and mdc_0_obuf= '0' then
          emac_mdio_in<= MDIO_IN_0 ;
        end if;
    end process mdio_reg2;


    --********** EMAC0 **********
    ----------------------------------------------------------------------------
    -- GTX_CLK Clock Management - 125 MHz clock frequency supplied by the user
    ----------------------------------------------------------------------------

    gtx : BUFG
    port map (
        I => gtx_clk_bufg_in,
        O => gtx_clk_bufg_0_i
        );

    tx_gmii_mii_clk_0_bufg : BUFG
    port map (
        I => tx_gmii_mii_clk_out_0_i,
        O => tx_gmii_mii_clk_in_0_i
        );

    ----------------------------------------------------------------------------
    -- GMII Transmitter Clock Management
    ----------------------------------------------------------------------------
    gmii_tx_clk_0_obuf : OBUF 
    port map (
        I => gmii_tx_clk_0_i, 
        O => gmii_tx_clk
        );

    ----------------------------------------------------------------------------
    -- GMII Transmitter Logic : Drive TX signals through IOBs onto GMII
    -- interface
    ----------------------------------------------------------------------------
    -- Infer IOB Output flip-flops.
    gmii_output_ffs_0 : process (tx_gmii_mii_clk_in_0_i, reset_i)
    begin
        if reset_i = '1' then
            gmii_tx_en_0_r <= '0';
            gmii_tx_er_0_r <= '0';
            gmii_txd_0_r   <= (others => '0');
        elsif tx_gmii_mii_clk_in_0_i'event and tx_gmii_mii_clk_in_0_i = '1' then
            gmii_tx_en_0_r <= gmii_tx_en_0_i;
            gmii_tx_er_0_r <= gmii_tx_er_0_i;
            gmii_txd_0_r   <= gmii_txd_0_i;
        end if;
    end process gmii_output_ffs_0;

    -- Drive GMII TX signals through Output Buffers and onto PADS
    gmii_tx_en_0_obuf : OBUF port map (I => gmii_tx_en_0_r, O => gmii_tx_en);
    gmii_tx_er_0_obuf : OBUF port map (I => gmii_tx_er_0_r, O => gmii_tx_er);
    gmii_txd_0_bus: for I in 7 downto 0 generate
        gmii_txd_0_obuf : OBUF 
        port map (
            I => gmii_txd_0_r(I),
            O => gmii_txd(I)
            );
    end generate;

    ----------------------------------------------------------------------------
    -- GMII Receiver Clock Management
    ----------------------------------------------------------------------------
    gmii_rx_clk_0_ibufg : IBUFG
    port map (
        I => gmii_rx_clk,
        O => gmii_rx_clk_ibufg_0_i
        );

    gmii_rx_clk_0_bufg : BUFG
    port map (
        I => gmii_rx_clk_ibufg_0_i,
        O => gmii_rx_clk_0_i
       );

    ----------------------------------------------------------------------------
    -- GMII Receiver Logic : Receive RX signals through IOBs from GMII
    -- interface
    ----------------------------------------------------------------------------
    -- Drive input GMII Rx signals from PADS through Input Buffers and then 
    -- use IDELAYs to provide Zero-Hold Time Delay ?!?!?
    -- Should the C, CE, INC, and RST ports be connected to GND or left OPEN ?!
    gmii_rx_dv_0_ibuf: IBUF port map (I => gmii_rx_dv, O => gmii_rx_dv_0_i);

    gmii_rx_dv_0_delay : IDELAY
    port map (
        I   => gmii_rx_dv_0_i,
        O   => gmii_rx_dv_delay_0_i,
        C   => gnd_i,
        CE  => gnd_i,
        INC => gnd_i,
        RST => gnd_i
        );

    gmii_rx_er_0_ibuf: IBUF port map (I => gmii_rx_er, O => gmii_rx_er_0_i);

    gmii_rx_er_0_delay : IDELAY
    port map (
        I   => gmii_rx_er_0_i,
        O   => gmii_rx_er_delay_0_i,
        C   => gnd_i,
        CE  => gnd_i,
        INC => gnd_i,
        RST => gnd_i
        );

    gmii_rxd_0_bus: for I in 7 downto 0 generate
        gmii_rxd_0_ibuf: IBUF
        port map (
            I => gmii_rxd(I),
            O => gmii_rxd_0_i(I)
            );
        gmii_rxd_0_delay : IDELAY
        port map (
            I   => gmii_rxd_0_i(I),
            O   => gmii_rxd_delay_0_i(I),
            C   => gnd_i,
            CE  => gnd_i,
            INC => gnd_i,
            RST => gnd_i
            );
    end generate;

    -- Infer IOB Input flip-flops
    gmii_input_ffs_0 : process (gmii_rx_clk_0_i, reset_i)
    begin
        if reset_i = '1' then
            gmii_rx_dv_0_r <= '0';
            gmii_rx_er_0_r <= '0';
            gmii_rxd_0_r   <= (others => '0');
        elsif gmii_rx_clk_0_i'event and gmii_rx_clk_0_i = '1' then
            gmii_rx_dv_0_r <= gmii_rx_dv_delay_0_i;
            gmii_rx_er_0_r <= gmii_rx_er_delay_0_i;
            gmii_rxd_0_r   <= gmii_rxd_delay_0_i;
        end if;
    end process gmii_input_ffs_0;

    -- MII_TX_CLK
    mii_tx_clk_0_ibufg : IBUFG 
    port map (
        I => mii_tx_clk,
        O => mii_tx_clk_0_i
        );



    -- Half Duplex signals for tri-speed mode
    gmii_col_0_obuf : IBUF port map (I => gmii_col, O => gmii_col_0_i);
    gmii_crs_0_obuf : IBUF port map (I => gmii_crs, O => gmii_crs_0_i);



    ----------------------------------------------------------------------------
    -- Receive & Transmit Client Clock Management when configured in 8-bit mode
    ----------------------------------------------------------------------------

    rx_client_clk_0_bufg : BUFG
    port map (
        I => rx_client_clk_out_0_i,
        O => rx_client_clk_in_0_i
        );

    tx_client_clk_0_bufg : BUFG
    port map (
        I => tx_client_clk_out_0_i,
        O => tx_client_clk_in_0_i
        );



    rx_clk <= rx_client_clk_in_0_i;
    tx_clk <= tx_client_clk_in_0_i;
    ----------------------------------------------------------------------------
    -- Instantiate V4 EMAC
    ----------------------------------------------------------------------------
    v4_emac_top : temac
    port map (
        -- Client Receiver Interface - EMAC0
        EMAC0CLIENTRXCLIENTCLKOUT       => rx_client_clk_out_0_i,
        CLIENTEMAC0RXCLIENTCLKIN        => rx_client_clk_in_0_i,
        EMAC0CLIENTRXD                  => rx_data,
        EMAC0CLIENTRXDVLD               => rx_valid,
        EMAC0CLIENTRXDVLDMSW            => open,
        EMAC0CLIENTRXGOODFRAME          => rx_good_frame,
        EMAC0CLIENTRXBADFRAME           => rx_bad_frame,
        EMAC0CLIENTRXFRAMEDROP          => open,
        EMAC0CLIENTRXDVREG6             => open,
        EMAC0CLIENTRXSTATS              => open,
        EMAC0CLIENTRXSTATSVLD           => open,
        EMAC0CLIENTRXSTATSBYTEVLD       => open,

        -- Client Transmitter Interface - EMAC0
        EMAC0CLIENTTXCLIENTCLKOUT       => tx_client_clk_out_0_i,
        CLIENTEMAC0TXCLIENTCLKIN        => tx_client_clk_in_0_i,
        CLIENTEMAC0TXD                  => tx_data,
        CLIENTEMAC0TXDVLD               => tx_valid,
        CLIENTEMAC0TXDVLDMSW            => '0',
        EMAC0CLIENTTXACK                => tx_ack,
        CLIENTEMAC0TXFIRSTBYTE          => '0',
        CLIENTEMAC0TXUNDERRUN           => '0',
        EMAC0CLIENTTXCOLLISION          => open,
        EMAC0CLIENTTXRETRANSMIT         => open,
        CLIENTEMAC0TXIFGDELAY           => "00000000",
        EMAC0CLIENTTXSTATS              => open,
        EMAC0CLIENTTXSTATSVLD           => open,
        EMAC0CLIENTTXSTATSBYTEVLD       => open,

        -- MAC Control Interface - EMAC0
        CLIENTEMAC0PAUSEREQ             => '0',
        CLIENTEMAC0PAUSEVAL             => "0000000000000000",

        -- Clock Signal - EMAC0
        GTX_CLK_0                       => gtx_clk_bufg_0_i,
        EMAC0CLIENTTXGMIIMIICLKOUT      => tx_gmii_mii_clk_out_0_i,
        CLIENTEMAC0TXGMIIMIICLKIN       => tx_gmii_mii_clk_in_0_i,

        -- GMII Interface - EMAC0
        GMII_TXD_0                      => gmii_txd_0_i,
        GMII_TX_EN_0                    => gmii_tx_en_0_i,
        GMII_TX_ER_0                    => gmii_tx_er_0_i,
        GMII_TX_CLK_0                   => gmii_tx_clk_0_i,
        GMII_RXD_0                      => gmii_rxd_0_r,
        GMII_RX_DV_0                    => gmii_rx_dv_0_r,
        GMII_RX_ER_0                    => gmii_rx_er_0_r,
        GMII_RX_CLK_0                   => gmii_rx_clk_0_i,
        MII_TX_CLK_0                    => mii_tx_clk_0_i,
        GMII_COL_0                      => gmii_col_0_i,
        GMII_CRS_0                      => gmii_crs_0_i,


        -- MDIO Interface - EMAC0
        MDC_0                           => emac_mdc,
        MDIO_IN_0                       => emac_mdio_in,
        MDIO_OUT_0                      => emac_mdio_out,
        MDIO_TRI_0                      => emac_mdio_tri,

        HOSTCLK                         => sys_clk,

        -- Asynchronous Reset
        RESET                           => reset_i
        );




-- DCM providing 125MHz gtx_clk for temac
   dcm1 : DCM
   generic map(
   CLKIN_PERIOD       => 10,
   DFS_FREQUENCY_MODE => "LOW",
   CLKFX_DIVIDE       => 4,
   CLKFX_MULTIPLY     => 5,
   CLK_FEEDBACK       => "1X"
   )
   port map(
     CLK0     => dcm1_clk0,    
     CLK180   => open,
     CLK270   => open,
     CLK2X    => open,
     CLK2X180 => open,
     CLK90    => open,
     CLKDV    => open,
     CLKFB    => dcm1_clk0,       
     CLKFX    => gtx_clk_bufg_in,    
     CLKFX180 => open,
     CLKIN    => clk_100mhz,     
     DSSEN    => '0',
     LOCKED   => dcm1_locked, 
     PSCLK    => '0',        
     PSDONE   => open, 
     PSEN     => '0',        
     PSINCDEC => '0',        
     RST      => '0',   
     STATUS   => open
     );

----sys_rst--------------------------------------------------------------
-- FIFO16: Virtex-4 1k deep x 18 wide BlockRAM Asynchrnous FIFO
------------------------------------------------------------------
rx_fifo : FIFO16
generic map (
    ALMOST_FULL_OFFSET      => X"010", 
    ALMOST_EMPTY_OFFSET     => X"010",
    DATA_WIDTH              => 9, 
    FIRST_WORD_FALL_THROUGH => FALSE) 

port map (
    ALMOSTEMPTY      => open, 
    ALMOSTFULL       => open, 
    DO (31 DOWNTO 8) => OPEN, 
    DO (7 DOWNTO 0)  => rx_fifo_DO,
    DOP (3 DOWNTO 1) => OPEN, 
    DOP(0)           => rx_valid_fifo,
    EMPTY            => open,
    FULL             => rx_fifo_full, 
    RDCOUNT          => open,
    RDERR            => rx_fifo_rderr,
    WRCOUNT          => OPEN,
    WRERR            => OPEN,
    DI (31 DOWNTO 8) => "000000000000000000000000",
    DI (7 DOWNTO 0)  => rx_data, 
    DIP (3 DOWNTO 1) => "000",
    DIP(0)           => rx_valid,
    RDCLK            => sys_clk,
    RDEN             => rx_fifo_rden,
    RST              => rx_fifo_rst,
    WRCLK            => rx_clk, 
    WREN             => rx_fifo_wren
);





tx_fifo : FIFO16
generic map (
    ALMOST_FULL_OFFSET => X"010", 
    DATA_WIDTH => 9, 
    FIRST_WORD_FALL_THROUGH => false ) 

port map (
    ALMOSTEMPTY         => OPEN,
    ALMOSTFULL          => open,
    DO (31 DOWNTO 8)    => OPEN, 
    DO (7 DOWNTO 0)     => tx_data,
    DOP (3 DOWNTO 1)    => OPEN, 
    DOP (0)             => tx_valid,
    EMPTY               => tx_fifo_empty,
    FULL                => tx_fifo_full_wire, 
    RDCOUNT(11 downto 0)=> open,
    RDERR               => open, 
    WRCOUNT             => OPEN, 
    WRERR               => open, 
    DI (31 DOWNTO 8)    => "000000000000000000000000", 
    DI (7 DOWNTO 0)     => tx_fifo_di(7 downto 0),
    DIP (3 DOWNTO 1)    => "000", 
    DIP(0)              => n_tx_fifo_di8,
    RDCLK               => tx_clk,
    RDEN                => tx_fifo_rden,
    RST                 => sys_rst, 
    WRCLK               => sys_clk,
    WREN                => tx_fifo_wren
);
n_tx_valid     <= not tx_valid;
n_tx_fifo_di8  <= not tx_fifo_di(8);
end architecture STRUCTURE;





