////////////////////////////////////////////////////////////////////////////////
// Module: temac_ultracontroller_module - cache only
////////////////////////////////////////////////////////////////////////////////
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION \"AS IS\"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
// 
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Filename:     top_temac_example.v
// 
// Description:    
// Module instantiates an UltraController2 and temac_controller for Demonstration
// of TEMAC UltraController Module
//
// Design Notes:
//   
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// Author:      APD/SEG         
// History:     2005.8.10 - Initial release
// 
//
////////////////////////////////////////////////////////////////////////////////

module top_temac_example (
            sys_clk, 
            nsys_rst, 
            gmii_col,
            gmii_crs,
            gmii_rx_clk,
            gmii_rx_dv,
            gmii_rx_er,
            gmii_rxd , 
            mii_tx_clk, 
            gmii_tx_clk,
            gmii_tx_en,
            gmii_tx_er,
            gmii_txd ,
            MDC_0,
            mdio,
            phy_mii_int_n,
            phy_rst_n,
            led_e,
            led_c,
            led_w,
            lcd_e,
            lcd_rs,
            lcd_rw,
            lcd_db
);

   input  sys_clk; 
   input  nsys_rst; 
   input  gmii_col;
   input  gmii_crs;
   input  gmii_rx_clk;
   input  gmii_rx_dv;
   input  gmii_rx_er;
   input  [0:7] gmii_rxd; 
   input  mii_tx_clk;
   output gmii_tx_clk;
   output gmii_tx_en;
   output gmii_tx_er;
   output [0:7] gmii_txd; 
   output MDC_0; 
   inout  mdio;
   input  phy_mii_int_n;
   output phy_rst_n ;
   output led_e;
   output led_c;
   output led_w;
   output lcd_e;
   output lcd_rs;
   output lcd_rw;
   output [7:4] lcd_db;


// common jtag signals
   wire tck; 
   wire tms;

// jtagppc signals to PPC405
   wire tdi_ppc_1; 
   wire tdo_ppc_1; 
   wire tdoen_ppc_1;
//   wire tdo_ts_ppc; 

// uc2 clock, reset and interrupt signals
   wire uc_sys_clk; 
   wire uc_sys_clk_i;
   wire sys_rst; 
   wire sys_rst_out; 

// uc2 gpio inputs and outputs signals     
   wire[0:31] gpio_out_s; 
   wire[0:31] gpio_in_s;


   wire [0:31] DSOCMBRAMWRDBUS; 
   wire DSOCMRDADDRVALID;
   wire DSOCMWRADDRVALID;
   wire dcm1_locked;
   wire dcm2_locked;
   wire dcm3_locked;
   wire dcm_locked; 
   wire tx_fifo_full;
   wire phy_mii_int;
   wire uc_sys_clk_bufg_o;
   wire dcm2_clkfx;
   wire dcm2_clk0;
   wire proc_clk;
   wire proc_clk_i;
   wire uar_done;
   wire load_clk;

   assign sys_rst         = ~nsys_rst | ~dcm_locked | sys_rst_out;   
   assign phy_rst_n       = gpio_out_s[1];

   assign led_e           = dcm1_locked;
   assign led_c           = dcm2_locked;
   assign led_w           = dcm3_locked;

   assign lcd_e           = gpio_out_s[25];
   assign lcd_rs          = gpio_out_s[26];
   assign lcd_rw          = gpio_out_s[27];
   assign lcd_db[7:4]     = gpio_out_s[28:31];

   assign dcm_locked      = dcm1_locked & dcm2_locked & dcm3_locked;

////////////////////////////////////////////////////////////////////////////////
//  BUFG  
////////////////////////////////////////////////////////////////////////////////
   BUFG BUFG_U1 (
       .I (sys_clk), 
       .O (uc_sys_clk_bufg_o)
    ); 

   DCM DCM2(
     .CLK0     (dcm2_clk0),
     .CLK180   (),
     .CLK270   (),
     .CLK2X    (),
     .CLK2X180 (),
     .CLK90    (),
     .CLKDV    (),
     .CLKFB    (dcm2_clk0),
     .CLKFX    (dcm2_clkfx),    
     .CLKFX180 (),
     .CLKIN    (uc_sys_clk_bufg_o),     
     .DSSEN    (1'b0),
     .LOCKED   (dcm2_locked), 
     .PSCLK    (1'b0),        
     .PSDONE   (), 
     .PSEN     (1'b0),        
     .PSINCDEC (1'b0),        
     .RST      (),   
     .STATUS   ()
     );

   defparam DCM2.CLKIN_PERIOD       = 10;
   defparam DCM2.DFS_FREQUENCY_MODE = "LOW";
   defparam DCM2.CLKFX_MULTIPLY     = 3;
   defparam DCM2.CLKFX_DIVIDE       = 2;
   defparam DCM2.CLK_FEEDBACK       = "1X"; 


   DCM DCM3(
     .CLK0     (uc_sys_clk_i),    
     .CLK180   (),
     .CLK270   (),
     .CLK2X    (proc_clk_i),
     .CLK2X180 (),
     .CLK90    (),
     .CLKDV    (load_clk),
     .CLKFB    (uc_sys_clk),       
     .CLKFX    (),    
     .CLKFX180 (),
     .CLKIN    (dcm2_clkfx),     
     .DSSEN    (1'b0),
     .LOCKED   (dcm3_locked), 
     .PSCLK    (1'b0),        
     .PSDONE   (), 
     .PSEN     (1'b0),        
     .PSINCDEC (1'b0),        
     .RST      (~dcm2_locked),   
     .STATUS   ()
     );

     defparam DCM3.DLL_FREQUENCY_MODE = "LOW";
     defparam DCM3.CLKIN_PERIOD       = 6.6666667;
     defparam DCM3.CLKDV_DIVIDE       = 6;
     defparam DCM3.CLK_FEEDBACK       = "1X"; 

   BUFG BUFG_PROC_CLK (
       .I (proc_clk_i), 
       .O (proc_clk)
       ); 


   BUFG BUFG_SYS_CLK (
       .I (uc_sys_clk_i), 
       .O (uc_sys_clk)
       ); 

////////////////////////////////////////////////////////////////////////////////
// UltraController2 
////////////////////////////////////////////////////////////////////////////////
   uc2 UltraController2 (
                         .sys_rst            (sys_rst), 
                         .sys_clk            (uc_sys_clk), 
                         .gpio_out           (gpio_out_s), 
                         .gpio_in            (gpio_in_s), 
                         .interrupt          (phy_mii_int),    
                         .sys_rst_out        (sys_rst_out), 
                         .jtag_tck           (tck), 
                         .jtag_tms           (tms), 
                         .jtag_tdi           (tdi_ppc_1), 
                         .jtag_tdo           (tdo_ppc_1), 
                         .jtag_tdoen         (tdoen_ppc_1),
                         .DSOCMBRAMWRDBUS    (DSOCMBRAMWRDBUS), 
                         .DSOCMRDADDRVALID   (DSOCMRDADDRVALID),
                         .DSOCMWRADDRVALID   (DSOCMWRADDRVALID),
                         .tx_fifo_full       (tx_fifo_full),
                         .proc_clk           (proc_clk)
     ); 

///////////////////////////////////////////////////////////////////////////////////
//temac_controller
//////////////////////////////////////////////////////////////////////////////////
     temac_controller temac_c(
        .gmii_col       (gmii_col), 
        .gmii_crs       (gmii_crs),
        .gmii_rx_clk    (gmii_rx_clk),
        .gmii_rx_dv     (gmii_rx_dv),
        .gmii_rx_er     (gmii_rx_er), 
        .gmii_rxd       (gmii_rxd),
        .mii_tx_clk     (mii_tx_clk),
        .gmii_tx_clk    (gmii_tx_clk),
        .gmii_tx_en     (gmii_tx_en),
        .gmii_tx_er     (gmii_tx_er),
        .gmii_txd       (gmii_txd), 
        .mdc_0          (MDC_0),
        .phy_mii_int_n  (phy_mii_int_n),
        .phy_mii_int    (phy_mii_int),
        .mdio           (mdio),
        .sys_clk        (uc_sys_clk),
        .dcm1_locked    (dcm1_locked),
        .gpio_in        (gpio_in_s),
        .rx_fifo_rst    (gpio_out_s[0]),
        .tx_fifo_lock_n (gpio_out_s[2]),
        .rx_fifo_rden   (DSOCMRDADDRVALID),
        .tx_fifo_wren   (DSOCMWRADDRVALID),
        .tx_fifo_full   (tx_fifo_full),
        .DSOCMBRAMWRDBUS(DSOCMBRAMWRDBUS),
        .sys_rst        (sys_rst),
        .clk_100mhz     (uc_sys_clk_bufg_o)
        );



////////////////////////////////////////////////////////////////////////////////
// JTAG signals
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// cache-loading through USR_ACCESS_VIRTEX4 register
////////////////////////////////////////////////////////////////////////////////
uar_load uar_load_inst (  .clk(uc_sys_clk_bufg_o), 
                          .rst(sys_rst), 
                          .tck(tck),
                          .tms(tms), 
                          .tdi(tdi_ppc_1), 
                          .uar_data_out(), 
                          .uar_datavalid(), 
                          .uar_done(uar_done) );

STARTUP_VIRTEX4 STARTUP_VIRTEX4_inst (
                              .CLK(),
                              .GSR(),
                              .GTS(),
                              .USRCCLKO(load_clk),
                              .USRCCLKTS(sys_rst),
                              .USRDONEO(uar_done),
                              .USRDONETS(1'b0),
                              .EOS()
  );

///////////////////////////////////////////////////////////////////////////////
// cache-loading through JTAGPPC (e.g. System ACE CF)
///////////////////////////////////////////////////////////////////////////////

//   JTAGPPC JTAGPPC_I (
//                    .TCK      (tck), 
//                    .TMS      (tms), 
//                    .TDIPPC   (tdi_ppc_1), 
//                    .TDOPPC   (tdo_ppc_1),
//                    .TDOTSPPC (tdo_ts_ppc)
//                    );
//   
//   assign tdo_ts_ppc = tdoen_ppc_1 ;


endmodule
