--------------------------------------------------------------------------------
--
--//////////////////////////////////////////////////////////////////////////////
-- Module: top_temac_example - cache only
--//////////////////////////////////////////////////////////////////////////////
--
--     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
--     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
--     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
--     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
--     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
--     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
--     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
--     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
--     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
--     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
--     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
--     FOR A PARTICULAR PURPOSE.
--     
--     (c) Copyright 2005 Xilinx, Inc.
--     All rights reserved.
-- 
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Filename:     top_temac_example.vhd
-- 
-- Description:    
-- Module instantiates an UltraController2 and temac_controllerfor Demonstration
-- of TEMAC UltraController Module
--
-- Design Notes:
--   
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Author:      SEG         
-- History:     2005.8.10 - Initial release
-- 
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;


entity top_temac_example is
  port (
    sys_clk       : in  std_logic;
    nsys_rst      : in  std_logic;
    gmii_col      : in std_logic;
    gmii_crs      : in std_logic;
    gmii_rx_clk   : in std_logic;
    gmii_rx_dv    : in std_logic;
    gmii_rx_er    : in std_logic;
    gmii_rxd      : in std_logic_vector (0 to 7); 
    mii_tx_clk    : in std_logic; 
    gmii_tx_clk   : out std_logic;
    gmii_tx_en    : out std_logic;
    gmii_tx_er    : out std_logic;
    gmii_txd      : out std_logic_vector(0 to 7);
    MDC_0         : out std_logic;
    mdio          : inout std_logic;
    phy_mii_int_n : in  std_logic;
    phy_rst_n     : out std_logic;
    led_e         : out std_logic;
    led_c         : out std_logic;
    led_w         : out std_logic;
    lcd_e         : out std_logic;
    lcd_rs        : out std_logic;
    lcd_rw        : out std_logic;
    lcd_db        : out std_logic_vector (7 downto 4)
    );
end entity top_temac_example;

architecture structure of top_temac_example is
-------------------------------------------------------------------
-- UltraController2 
-------------------------------------------------------------------
  component uc2
    port (
      sys_clk          : in  std_logic;
      sys_rst          : in  std_logic;
      gpio_out         : out std_logic_vector(0 to 31);
      gpio_in          : in  std_logic_vector(0 to 31);
      interrupt        : in  std_logic;
      sys_rst_out      : out std_logic;
      jtag_tck         : in  std_logic;
      jtag_tms         : in  std_logic;
      jtag_tdi         : in  std_logic;
      jtag_tdo         : out std_logic;
      jtag_tdoen       : out std_logic;
      DSOCMBRAMWRDBUS  : out std_logic_vector (0 to 31); 
      DSOCMRDADDRVALID : out std_logic;
      DSOCMWRADDRVALID : out std_logic;
      tx_fifo_full     : in std_logic;
      proc_clk         : in std_logic
 );
  end component;
component temac_controller 
  port (
    gmii_col          : in std_logic;
    gmii_crs          : in std_logic;
    gmii_rx_clk       : in std_logic;
    gmii_rx_dv        : in std_logic;
    gmii_rx_er        : in std_logic;
    gmii_rxd          : in std_logic_vector (0 to 7);
    mii_tx_clk        : in std_logic ;
    gmii_tx_clk       : out std_logic;
    gmii_tx_en        : out std_logic;
    gmii_tx_er        : out std_logic;
    gmii_txd          : out std_logic_vector(0 to 7); 
    phy_mii_int_n     : in  std_logic;
    phy_mii_int       : out std_logic;
    MDC_0             : out std_logic;
    mdio              : inout std_logic;
    sys_clk           : in std_logic;
    dcm1_locked       : out std_logic;
    gpio_in           : out std_logic_vector (0 to 31);
    rx_fifo_rst       : in std_logic;
    tx_fifo_lock_n    : in std_logic;
    rx_fifo_rden      : in std_logic;
    tx_fifo_wren      : in std_logic;
    tx_fifo_full      : out std_logic;
    DSOCMBRAMWRDBUS   : in std_logic_vector (0 to 31);
    sys_rst           : in std_logic;
    clk_100mhz        : in std_logic );
end component;


component DCM is
generic (
    DLL_FREQUENCY_MODE  : string  := "LOW";
    CLKIN_PERIOD        : real    := 10.0;
    DFS_FREQUENCY_MODE  : string  := "LOW";
    CLKFX_DIVIDE        : integer := 1;
    CLKFX_MULTIPLY      : integer := 2;
    CLKDV_DIVIDE        : integer := 2;
    CLK_FEEDBACK        : string  := "NONE" 
);
port
    (
     CLK0     : out std_logic;      
     CLK180   : out std_logic;  
     CLK270   : out std_logic;  
     CLK2X    : out std_logic;  
     CLK2X180 : out std_logic;  
     CLK90    : out std_logic;  
     CLKDV    : out std_logic;    
     CLKFB    : in  std_logic;    
     CLKFX    : out std_logic;
     CLKFX180 : out std_logic;      
     CLKIN    : in  std_logic;     
     DSSEN    : in  std_logic;       
     LOCKED   : out std_logic;
     PSCLK    : in  std_logic;       
     PSDONE   : out std_logic;    
     PSEN     : in  std_logic;    
     PSINCDEC : in  std_logic;     
     RST      : in  std_logic;  
     STATUS   : out std_logic_vector (7 downto 0)   
     );
  end component;
attribute box_type of DCM: component is "black_box";

component uar_load is
port(
  clk           : in std_logic;
  rst           : in std_logic;
  tck           : out std_logic;
  tms           : out std_logic;
  tdi           : out std_logic;
  uar_data_out  : out std_logic_vector (31 downto 0);
  uar_datavalid : out std_logic;
  uar_done      : out std_logic
  );
end component;

-- common jtagppc signals
  signal tck : std_logic;
  signal tms : std_logic;

-- jtagppc signals to 1st PPC405 for 2VP4 - 2VP100s
  signal tdi_ppc_1   : std_logic;
  signal tdo_ppc_1   : std_logic;
  signal tdoen_ppc_1 : std_logic;

-- Enable this signal when using JTAGPPC instead of uar_load
--  signal tdo_ts_ppc  : std_logic;

-- uc2 clock, reset and interrupt signals
  signal uc_sys_clk  : std_logic;
  signal sys_rst     : std_logic;

-- uc2 gpio inputs and outputs signals  
  signal gpio_out_s  : std_logic_vector(0 to 31);
  signal gpio_in_s   : std_logic_vector(0 to 31);


  signal DSOCMBRAMWRDBUS  : std_logic_vector (0 to 31); 
  signal DSOCMRDADDRVALID : std_logic;
  signal DSOCMWRADDRVALID : std_logic;
  signal dcm1_locked      : std_logic;
  signal dcm2_locked      : std_logic;
  signal dcm3_locked      : std_logic;
  signal dcm_locked       : std_logic;
  signal tx_fifo_full     : std_logic;
  signal phy_mii_int      : std_logic;
  signal uar_done         : std_logic;
  signal load_clk         : std_logic;
  signal proc_clk         : std_logic;
  signal proc_clk_i       : std_logic;
  signal uc_sys_clk_bufg_o: std_logic;
  signal uc_sys_clk_i     : std_logic;
  signal dcm2_clkfx       : std_logic;
  signal dcm2_clk0        : std_logic;
  signal sys_rst_out      : std_logic;
  
begin
   phy_rst_n          <=  gpio_out_s(1);
   -- reset logic
   sys_rst            <= (not nsys_rst) or (not dcm_locked) or sys_rst_out;
   led_e              <= dcm1_locked;
   led_c              <= dcm2_locked;
   led_w              <= dcm3_locked;

   lcd_e              <= gpio_out_s(25);
   lcd_rs             <= gpio_out_s(26);
   lcd_rw             <= gpio_out_s(27);
   lcd_db(7 downto 4) <= gpio_out_s(28 to 31);

   dcm_locked         <= dcm1_locked and dcm2_locked and dcm3_locked;
-------------------------------------------------------------------
-- Instantiate the a BUFP to drive sys_clk
-------------------------------------------------------------------
   BUFG_U1 : BUFG port map (
    I => sys_clk,
    O => uc_sys_clk_bufg_o
    );


   dcm2 : DCM
   generic map(
      CLKIN_PERIOD         => 10.0,
      DFS_FREQUENCY_MODE   => "LOW",
      CLKFX_DIVIDE         => 2,
      CLKFX_MULTIPLY       => 3,
      CLK_FEEDBACK         => "1X"
    )
   port map(
     CLK0     => dcm2_clk0,    
     CLK180   => open,
     CLK270   => open,
     CLK2X    => open,
     CLK2X180 => open,
     CLK90    => open,
     CLKDV    => open,
     CLKFB    => dcm2_clk0,
     CLKFX    => dcm2_clkfx,    
     CLKFX180 => open,
     CLKIN    => uc_sys_clk_bufg_o,     
     DSSEN    => '0',
     LOCKED   => dcm2_locked, 
     PSCLK    => '0',        
     PSDONE   => open, 
     PSEN     => '0',        
     PSINCDEC => '0',        
     RST      => '0',   
     STATUS   => open
     );

   dcm3 : DCM
   generic map(
   DLL_FREQUENCY_MODE => "LOW",
   CLKIN_PERIOD       => 6.6666667,
   CLKDV_DIVIDE       => 6,
   CLK_FEEDBACK       => "1X"
  )
   port map(
     CLK0     => uc_sys_clk_i,    
     CLK180   => open,
     CLK270   => open,
     CLK2X    => proc_clk_i,
     CLK2X180 => open,
     CLK90    => open,
     CLKDV    => load_clk,
     CLKFB    => uc_sys_clk,       
     CLKFX    => open,    
     CLKFX180 => open,
     CLKIN    => dcm2_clkfx,     
     DSSEN    => '0',
     LOCKED   => dcm3_locked, 
     PSCLK    => '0',        
     PSDONE   => open, 
     PSEN     => '0',        
     PSINCDEC => '0',        
     RST      => (not dcm2_locked),  
     STATUS   => open
     );

   BUFG_PROC : BUFG port map (
    I => proc_clk_i,
    O => proc_clk
    );
   
   BUFG_SYS_CLK : BUFG port map (
    I => uc_sys_clk_i,
    O => uc_sys_clk
    );


-------------------------------------------------------------------
-- Instantiate the UltraController Core
-------------------------------------------------------------------
  UltraController2 : uc2
    port map (
      sys_rst           => sys_rst,
      sys_clk           => uc_sys_clk,
      gpio_out          => gpio_out_s,
      gpio_in           => gpio_in_s,
      interrupt         => phy_mii_int,    
      sys_rst_out       => sys_rst_out,
      jtag_tck          => tck,             
      jtag_tms          => tms,              
      jtag_tdi          => tdi_ppc_1,        
      jtag_tdo          => tdo_ppc_1,         
      jtag_tdoen        => tdoen_ppc_1,  
      DSOCMBRAMWRDBUS   => DSOCMBRAMWRDBUS, 
      DSOCMRDADDRVALID  => DSOCMRDADDRVALID,
      DSOCMWRADDRVALID  => DSOCMWRADDRVALID,
      tx_fifo_full      => tx_fifo_full,
      proc_clk          => proc_clk
      );
  
  temac_c : temac_controller
    port map (
    gmii_col        => gmii_col, 
    gmii_crs        => gmii_crs,
    gmii_rx_clk     => gmii_rx_clk,
    gmii_rx_dv      => gmii_rx_dv,
    gmii_rx_er      => gmii_rx_er, 
    gmii_rxd        => gmii_rxd,
    mii_tx_clk      => mii_tx_clk,
    gmii_tx_clk     => gmii_tx_clk,
    gmii_tx_en      => gmii_tx_en,
    gmii_tx_er      => gmii_tx_er,
    gmii_txd        => gmii_txd, 
    MDC_0           => MDC_0,
    phy_mii_int_n   => phy_mii_int_n,
    phy_mii_int     => phy_mii_int,
    mdio            => mdio,
    sys_clk         => uc_sys_clk,
    dcm1_locked     => dcm1_locked,
    gpio_in         => gpio_in_s,
    rx_fifo_rst     => gpio_out_s(0),
    tx_fifo_lock_n  => gpio_out_s(2),
    rx_fifo_rden    => DSOCMRDADDRVALID,
    tx_fifo_wren    => DSOCMWRADDRVALID,
    tx_fifo_full    => tx_fifo_full,
    DSOCMBRAMWRDBUS => DSOCMBRAMWRDBUS,
    sys_rst         => sys_rst,
    clk_100mhz      => uc_sys_clk_bufg_o
);




-------------------------------------------------------------------
-- Instantiate the JTAGPPC for use with one or two PPCs
-------------------------------------------------------------------
--  JTAGPPC_I : JTAGPPC
--    port map (
--      TCK      => tck,                  -- in from JTAGPPC
--      TMS      => tms,                  -- in from JTAGPPC
--      TDIPPC   => tdi_ppc_1,            -- to UC2 
--      TDOPPC   => tdo_ppc_1,            -- from UC2
--      TDOTSPPC => tdo_ts_ppc
--      );
--  tdo_ts_ppc <= tdoen_ppc_1;
--

  uar_load_inst: uar_load 
  port map(
                clk           => uc_sys_clk_bufg_o, 
                rst           => sys_rst,
                tck           => tck,
                tms           => tms,
                tdi           => tdi_ppc_1, 
                uar_data_out  => open, 
                uar_datavalid => open, 
                uar_done      => uar_done
               );
   
  STARTUP_VIRTEX4_inst:STARTUP_VIRTEX4  
  port map(
    CLK       => open,
    GSR       => open,
    GTS       => open,
    USRCCLKO  => load_clk,
    USRCCLKTS => sys_rst,
    USRDONEO  => uar_done,
    USRDONETS => '0',
    EOS       => open
  );



end architecture structure;
