////////////////////////////////////////////////////////////////////////////////
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005 Xilinx, Inc.
//     All rights reserved.
// 
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Filename:     lcd.c
// 
// Description:    
// Two Line by 16 Character LCD Display Driver
//   
//-----------------------------------------------------------------------------

#include "xbasic_types.h"
#include "xtime_l.h"

extern void GPIO_writeLCD( Xuint32 );
extern void usleep( unsigned int );

//=====================
//	TEST FUNCTIONS
//=====================
void XromLCDTest();
void LCDTestMenu();
void LCDTestInput(char ch);


//=====================
//	EXTERNAL FUNCTIONS
//=====================
void LCD_Init();
void LCD_Write(char * line1, char * line2);

//void XromLCDOn();
//void XromLCDOff();
//void XromLCDClear();

void XromMoveCursorHome();
void XromMoveCursorRight();
void XromMoveCursorLeft();

void XromLCDSetLine(int line);
void XromLCDPrintChar(char c);

//=====================
//	INTERNAL FUNCTIONS
//=====================
void XromInitInst(void);
void XromWriteInst(unsigned long inst1, unsigned long inst2);
void XromWriteData(unsigned long data1, unsigned long data2);

// global variables
// LCD parameters 
#define INIT_DELAY 10000   // usec delay timer during initialization,
                           // important to change if clock speed changes
#define INST_DELAY 5000    // usec delay timer between instructions
#define DATA_DELAY 2500    // usec delay timer between data

//==============================================================================
//
//				INTERNAL FUNCTIONS
//
//==============================================================================

void XromInitInst(void)
{
	//XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000003);
	GPIO_writeLCD( 0x00000003 );
	usleep(1);
		
	//XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000043); //set enable and data
	GPIO_writeLCD( 0x00000043 );
	usleep(1);

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000003);
	GPIO_writeLCD( 0x00000003 );
	usleep(INIT_DELAY);
}

void XromWriteInst(unsigned long inst1, unsigned long inst2)
{
	unsigned long printinst;

	printinst = 0x00000040 | inst1;
	
	//XGpio_mSetDataReg(LCD_BASEADDR, 1, inst1); //write data
	GPIO_writeLCD( inst1 );  // write data
	usleep(1);

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, printinst); //set enable
	GPIO_writeLCD( printinst );  // set enable
	usleep(1);
	
	//XGpio_mSetDataReg(LCD_BASEADDR, 1, inst1); //turn off enable
	GPIO_writeLCD( inst1 );  // turn off enable
	usleep(1);
	
	printinst = 0x00000040 | inst2;

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, printinst); //set enable and data
	GPIO_writeLCD( printinst );  // set enable and data
	usleep(1);

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, inst2); //turn off enable
	GPIO_writeLCD( inst2 );  // turn off enable
	usleep(INST_DELAY);	
}

void XromWriteData(unsigned long data1, unsigned long data2)
{
        unsigned long rs_data, enable_rs_data;
//	int busy=true;	

	rs_data = 0x00000020 | data1; //sets rs, data1
	enable_rs_data = 0x00000060 | data1;

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, rs_data); //write data, rs
	GPIO_writeLCD( rs_data );  // write data, rs
	usleep(1);

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, enable_rs_data); //set enable, keep data, rs
	GPIO_writeLCD( enable_rs_data );  // set enable, keep data, rs
	usleep(1);

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, rs_data); //turn off enable
	GPIO_writeLCD( rs_data );  // turn off enable
	usleep(1);
	
	rs_data = 0x00000020 | data2; //sets rs, data2
	enable_rs_data = 0x00000060 | data2; //sets rs, data2

//XGpio_mSetDataReg(LCD_BASEADDR, 1, enable_rs_data); //set enable, rs, data
	GPIO_writeLCD( enable_rs_data );  // set enable, rs, data
	usleep(1);

	//XGpio_mSetDataReg(LCD_BASEADDR, 1, rs_data); //turn off enable
	GPIO_writeLCD( rs_data );  // turn off enable
	usleep(DATA_DELAY);
}

//===============================================================================
//
//				EXTERNAL FUNCTIONS
//
//===============================================================================
/*
void XromLCDOn()
{
	//printf("DISPLAY ON\r\n");
	XromWriteInst(0x00000000, 0x0000000E);
}
*/
/*void XromLCDOff()
{
	//printf("DISPLAY OFF\r\n");
	XromWriteInst(0x00000000, 0x00000008);
}
*/
/*void XromLCDClear()
{
	//printf("DISPLAY CLEAR\r\n");
	XromWriteInst(0x00000000, 0x00000001);
	XromWriteInst(0x00000000, 0x00000010);
}
*/
void LCD_Init()
{
	//XGpio_mSetDataDirection(LCD_BASEADDR, 1, 0x00000000); //Sets CHAR LCD Reg to Write Mode
	//XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000000); //Zeroes CHAR LCD Reg

	GPIO_writeLCD( 0x00000000 ); // Zeroes CHAR LCD register

	//LCD INIT
	usleep(15000);	//After VCC>4.5V Wait 15ms to Init Char LCD
	XromInitInst();
	usleep(4100); //Wait 4.1ms

	XromInitInst();
	usleep(100); //Wait 100us
	XromInitInst();
	XromInitInst();

	//Function Set
	XromWriteInst(0x00000002, 0x00000002);
	
	//Display Off
	XromWriteInst(0x00000000, 0x00000008);

	//Display Clear
	XromWriteInst(0x00000000, 0x00000001);

	//Entry Mode Set
	XromWriteInst(0x00000000, 0x00000006);

	//Display On
	XromWriteInst(0x00000000, 0x0000000E);
}

void XromMoveCursorHome()
{
	XromWriteInst(0x00000000, 0x00000002);
}

void XromMoveCursorLeft()
{
	XromWriteInst(0x00000001, 0x00000000);
}

void XromMoveCursorRight()
{
	XromWriteInst(0x00000001, 0x00000004);
}

void XromLCDSetLine(int line)
{ 
	int i;

	if((line - 1)) {
		XromMoveCursorHome();
		for(i=0; i<40; i++)
			XromMoveCursorRight();
	}
	else
		XromMoveCursorHome();
}

void XromLCDPrintChar(char c)
{
	XromWriteData(((c >> 4) & 0x0000000F), (c & 0x0000000F));
}

void LCD_Write(char * line1, char * line2)
{
	int i=0;

	XromLCDSetLine(1);

	for(i=0; i<16; i++){
		if(line1[i])
			XromLCDPrintChar(line1[i]);
		else break;
	}	

	XromLCDSetLine(2);

	for(i=0; i<16; i++){
		if(line2[i])
			XromLCDPrintChar(line2[i]);
		else break;
	}	
        return;
}


