////////////////////////////////////////////////////////////////////////////////
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2005-2006 Xilinx, Inc.
//     All rights reserved.
// 
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// Filename:     emac_original.c
// 
// Description:   EMAC Interface Module for EMAC UltraController Module
// 
//-----------------------------------------------------------------------------




#include "xexception_l.h"
#include "xbasic_types.h"
#include "xpseudo_asm.h"
#include "xgpemac_l.h"

#define mac_addr0 11
#define mac_addr1 12
#define mac_addr2 13
#define mac_addr3 14
#define mac_addr4 15
#define mac_addr5 16

/* Address of the Ethernet Phy.
   ML403: 0
   ML405: 0 or 7
   ML410: 0 or 7
*/
#ifndef PHY_ADDR
#define PHY_ADDR          0
#endif

#define C_10MBPS          2
#define C_100MBPS         1
#define AUTO_NEGOTIATION  0
#define CONNECTION        AUTO_NEGOTIATION

#define RX_DATA_FILTER  0x000000ff
#define RX_VALID_FILTER 0x00100000
#define RX_FULL_FILTER  0x00040000
#define RX_EMPTY_FILTER 0x00800000


#define XEMAC_0 0
#define FIFOPORT  0x01000000
#define EMACDCR_BASEADDR 0x3fc


static volatile int phy_access_done = 0, phy_link_down = 0, phy_link_up = 0;


void emac_init(){
  unsigned int result;
  //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //Enable interrupt
  //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  XExc_Init(); 
  XExc_RegisterHandler(XEXC_ID_FIT_INT, 
                      (XExceptionHandler)FIT_Exception_Handler, 
                      (void *) NULL);
  XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT, 
                      (XExceptionHandler)Ext_Exception_Handler, 
                      (void *) NULL);
  XTime_TSRClearStatusBits(XREG_TSR_CLEAR_ALL);
  XExc_mEnableExceptions(XEXC_NON_CRITICAL);
  //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  //Set mdio interface 
  //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  XGPEmac_mWriteConfigReg ( EMACDCR_BASEADDR,
                            0,
                            XGP_E0_MGMTCONFIG,
                            XGP_MDIO_ENABLE_MASK | XGP_MDIO_CLOCK_DIV_100MHz
                          );
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // Reset PHY
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  reset_phy_ethernet();
  reset_rx_fifo();
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // Set A-N Advertisement Regs for Full Duplex modes ONLY
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  result = XGPEmac_mGetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 4);
  XGPEmac_mSetPcsReg(EMACDCR_BASEADDR,0,PHY_ADDR,4,((result&0x0000FF5F)|0x00000140));
  result = XGPEmac_mGetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 9);
  XGPEmac_mSetPcsReg(EMACDCR_BASEADDR,0,PHY_ADDR,9,((result&0x0000FEFF)|0x00000200));
  #if CONNECTION != AUTO_NEGOTIATION
    result = XGPEmac_mGetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 0);
    XGPEmac_mSetPcsReg(EMACDCR_BASEADDR,0,PHY_ADDR,0,(result & 0x00000100));
    result = XGPEmac_mGetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 0);
  #endif
  #if CONNECTION == C_100MBPS
  XGPEmac_mSetPcsReg(EMACDCR_BASEADDR,0,PHY_ADDR,0,(result | 0x0000a200));
  #endif
  #if CONNECTION == C_10MBPS
    XGPEmac_mSetPcsReg(EMACDCR_BASEADDR,0,PHY_ADDR,0,(result | 0x00008200));
  #endif
  #if CONNECTION == AUTO_NEGOTIATION
  result = XGPEmac_mGetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 0);
  XGPEmac_mSetPcsReg(EMACDCR_BASEADDR,0,PHY_ADDR,0,(result | 0x00009200));
  #endif

  LCD_Write("Waiting for link", "                ");
  phy_access_done = 0;
  // set up interrupt from the PHY 
  XGPEmac_mSetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 18, 0x6c00);
  while (phy_access_done == 0); // spinloop until PHY is setup
}



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
// exceptional handler is called when there's an interrupt from PHY
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void Ext_Exception_Handler (void *CallBackRef)
{
  unsigned int result;
  // read from PHY status register 
  result = XGPEmac_mGetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 19);
  // if there is a link, configure emac
  if ((result & 0x00000c00) == (0x00000c00)){
    phy_link_up = 1;
    configure_emac();
  }
  // if the link is down: display link down
  else if (result & 0x00000400){
    LCD_Write("Link down       ", "                ");
    phy_link_down = 0;
  }
  else {
    LCD_Write("Check Link by    ", "pinging          ");
    phy_access_done = 1;
  }
}
void configure_emac(void)
{
  unsigned int result;
  Xuint32 macAddr;
  result = XGPEmac_mGetPcsReg(EMACDCR_BASEADDR, 0, PHY_ADDR, 17);
  
  if ( (result&0xE000) == 0xA000) {
    XGPEmac_mWriteConfigReg(EMACDCR_BASEADDR, 0, XGP_E0_EMACCONFIG, 0x80000000);
    LCD_Write("1000 baseT      ", "full duplex link");
  }
  else if ((result&0xE000) == 0x6000) {
    XGPEmac_mWriteConfigReg(EMACDCR_BASEADDR,0,XGP_E0_EMACCONFIG,0x40000000);
    LCD_Write("100 baseT       ", "full duplex link");
  }
  else if ((result&0xE000) == 0x2000) {
    XGPEmac_mWriteConfigReg(EMACDCR_BASEADDR,0,XGP_E0_EMACCONFIG,0x00000000);
    LCD_Write("10 baseT        ", "full duplex link");
  }
  else {
    LCD_Write("Half duplex     ", "not supported   ");
    return ;
  } 
  //configure address filtering
  // set MAC address of TEMAC
  macAddr = (0x000000ff&((Xuint32)mac_addr0)) | (0x0000ff00&(((Xuint32)mac_addr1)<<8)) |(0x00ff0000&(((Xuint32)mac_addr2)<<16)) |(0xff000000&(((Xuint32)mac_addr3)<<24)); 
  XGPEmac_mWriteAFReg(EMACDCR_BASEADDR, 0, XGP_E0_UNICASTADDRW0, macAddr);
  macAddr = ((Xuint32)mac_addr4) | (((Xuint32)mac_addr5)<<8);
  XGPEmac_mWriteAFReg(EMACDCR_BASEADDR, 0, XGP_E0_UNICASTADDRW1, macAddr);
  XGPEmac_mWriteAFReg(EMACDCR_BASEADDR, 0, XGP_E0_GENCONFIG, 0x0);
}




  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // function: emac_send
  // sends a frame of length "length", stored in an array frame_pointer
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

void emac_send(Xuint8 frame_pointer[], int length){
  int x;
  lock_tx_fifo();
  stw(FIFOPORT, ((Xuint32) (frame_pointer[0])));
    for(x = 0; x < length; ++x) {
    stw(FIFO_PORT, ((Xuint32) (frame_pointer[x])));
    }
  stw(FIFOPORT, 0x00000100);
  stw(FIFOPORT, 0x00000100);
  unlock_tx_fifo();
}


  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  // function: emac_receive
  // 		   reads the next frame in RX_FIFO and stores it in frame_pointer[], 
  // 	         returns the length of the frame. 
  // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
int emac_read(Xuint8 frame_pointer[]){
  Xuint32 input;
  u16_t x;
  do {
    x=0;
    do { 
      input            = lwz(FIFOPORT);
      frame_pointer[x] = (u8_t)(input & RX_DATA_FILTER);
      if (!(input & RX_EMPTY_FILTER)) 
      {x++;}
    } while (((input & RX_VALID_FILTER) || (input & RX_EMPTY_FILTER)) && (!(input & RX_FULL_FILTER)));
    if ((input & RX_FULL_FILTER)){
      reset_rx_fifo();
    }
  } while ((input & RX_FULL_FILTER) || (x==0));
  return (x-1);
}
