/******************************************************************************
*                           ****************************
*                           ** Copyright Xilinx, Inc. **
*                           ** All rights reserved.   **
*                           ****************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2005 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/**
* 
* @file xgpemac_l.h
*
* This header file contains identifiers and low-level driver functions (or
* macros) that can be used to access the device.  High-level driver functions
* are defined in xgemac.h.
*
* <pre>
* MODIFICATION HISTORY
*
* Who  Date     Changes
* ---- -------- -----------------------------------------------
* sn  03/11/04 First release
*
* </pre>
*
******************************************************************************/

#ifndef XGPEMAC_L_H /* prevent circular inclusions */
#define XGPEMAC_L_H /* by using protection macros */

/***************************** Include Files *********************************/

#include "xbasic_types.h"
#include "sleep.h"

/************************** Constant Definitions *****************************/


 /** @name Address Definitions
 * @{
 */
#define XGP_HIF_BASEADDR   0x10C   /* DCR Host Interface address */
#define XGP_HIF_DATA_REG_MSW_OFFSET     (0x000)
#define XGP_HIF_DATA_REG_LSW_OFFSET     (0x001)
#define XGP_HIF_CNTL_REG_OFFSET         (0x002)
#define XGP_HIF_RDY_STATUS_OFFSET       (0x003)
/* @} */

/** @name  DCR Ready Status Register masks for EMAC1 and EMAC0 
 * @{
 */
#define XGP_DCR_RDYSTAT_CONFIG1_WRITE_MASK 0x4000
#define XGP_DCR_RDYSTAT_CONFIG1_READ_MASK 0x2000
#define XGP_DCR_RDYSTAT_AF1_WRITE_MASK 0x1000
#define XGP_DCR_RDYSTAT_AF1_READ_MASK 0x0800
#define XGP_DCR_RDYSTAT_MIIM1_WRITE_MASK 0x0400
#define XGP_DCR_RDYSTAT_MIIM1_READ_MASK 0x0200
#define XGP_DCR_RDYSTAT_STATS1_READ_MASK 0x0100
/* EMAC 0 */
#define XGP_DCR_RDYSTAT_CONFIG0_WRITE_MASK 0x0040
#define XGP_DCR_RDYSTAT_CONFIG0_READ_MASK 0x0020
#define XGP_DCR_RDYSTAT_AF0_WRITE_MASK 0x0010
#define XGP_DCR_RDYSTAT_AF0_READ_MASK 0x0008
#define XGP_DCR_RDYSTAT_MIIM0_WRITE_MASK 0x0004
#define XGP_DCR_RDYSTAT_MIIM0_READ_MASK 0x0002
#define XGP_DCR_RDYSTAT_STATS0_READ_MASK 0x0001
/* @} */

/** @name DCR control register CntlReg masks
 * @{ 
 * */
#define XGP_DCR_CNTLREG_WRITE_ENABLE_MASK 0x8000 /* Write enable */
#define XGP_DCR_CNTLREG_EMAC1SEL_MASK 0x0400     /* EMAC1 or EMAC2 */
#define XGP_DCR_CNTLREG_ADDRESSCODE_MASK 0x3ff   /* 10 bit address code */
/* @} */

/** @name GEMAC/PHY configuration and stats 
* @{ 
* 1st channel */
#define XGP_E0_STATS0 0x0        /* frames received ok */
#define XGP_E0_STATS1 0x1        /* FCS errors */
#define XGP_E0_STATS2 0x2        /* Broadcast frames received ok */
#define XGP_E0_STATS3 0x3        /* Multicast frames received ok */
#define XGP_E0_STATS4 0x4        /* 64 byte frames received ok */

#define XGP_E0_RXCONFIGW0 0x200  /* Receiver configuration word 0 */
#define XGP_E0_RXCONFIGW1 0x240  /* Receiver configuration word 1 */
#define XGP_E0_TXCONFIG 0x280    /* Transmitter configuration */
#define XGP_E0_FLOWCONTROL 0x2C0 /* Flow control configuration */
#define XGP_E0_EMACCONFIG 0x300  /* Emac configuration */
#define XGP_E0_RGMIISGMII 0x320  /* RGMII/SGMII status */
#define XGP_E0_MGMTCONFIG 0x340  /* Management configuration */

#define XGP_E0_UNICASTADDRW0 0x380       /* Unicast Address [31:0] */
#define XGP_E0_UNICASTADDRW1 0x384       /* Unicast Address [47:32] */
#define XGP_E0_ADDRTABLECONFIGW0 0x388   /* CAM data [31:0] */
#define XGP_E0_ADDRTABLECONFIGW1 0x38C   /* CAMrnw, {000}, CAM Address [3:0], CAM data [47:32] */
#define XGP_E0_GENCONFIG 0x390           /* Reset, Promiscuous Mode */

#define XGP_IRSTATUS 0x3A0       /* Interrupt Request status */
#define XGP_IRENABLE 0x3A4       /* Interrupt Request enable */
#define XGP_MIIMWRDATA 0x3b0     /* MIIM write data */
#define XGP_MIIMCNTL 0x3b4       /* Decode address for MIIM address output */
/* @} */

/** @name Tx and Rxconfig word masks 
 * @{
 * */
/* TX config word masks */
#define XGP_E0_TXCONFIG_JUMBO_MASK    0x40000000
#define XGP_E0_TXCONFIG_TXENABLE_MASK 0x10000000
#define XGP_EO_TXCONFIG_VLAN_MASK     0x08000000

/* Rx config word masks */
#define XGP_E0_RXCONFIGW1_JUMBO_MASK     0x40000000
#define XGP_EO_RXCONFIGW1_RXENABLE_MASK  0x10000000
#define XGP_EO_RXCONFIGW1_VLAN_MASK      0x08000000
/* @} */
 

/** @name PCS registers 
 * @{
 * Register addresses */
#define XGP_PCS_CTRL_REG 0x0
#define XGP_PCS_STATUS_REG 0x1
#define XGP_PCS_PHY_ID1 0x2
#define XGP_PCS_PHY_ID2 0x3
#define XGP_PCS_AN_ADV_REG 0x4
#define XGP_PCS_AN_LINK_PART_REG 0x5
#define XGP_PCS_AN_EXP_REG 0x6
#define XGP_PCS_AN_NP_TX_REG 0x7
#define XGP_PCS_AN_NP_RX_REG 0x8
#define XGP_PCS_EXT_STATUS_REG 0xF
/* @} */


/** @name PCS Control register masks 
 * @{
 * */
#define XGP_PCS_CTRL_RESET_MASK 0x8000
#define XGP_PCS_CTRL_LOOPBACK_MASK 0x4000
#define XGP_PCS_CTRL_SPEEDSEL_LSB_MASK 0x2000
#define XGP_PCS_CTRL_AUTONEG_ENABLE_MASK 0x1000
#define XGP_PCS_CTRL_POWERDOWN_MASK 0x0800
#define XGP_PCS_CTRL_ISOLATE_MASK 0x0400
#define XGP_PCS_CTRL_AUTONEG_RESTART_MASK 0x0200
#define XGP_PCS_CTRL_DUPLEX_MASK 0x0100
#define XGP_PCS_CTRL_COLL_TEST_MASK 0x0080
#define XGP_PCS_CTRL_SPEEDSEL_MSB_MASK 0x0040
/* @} */


/** @name MDIO configuration word register masks 
 * @{ 
 * */
#define XGP_MDIO_ENABLE_MASK  0x40  /* enable MII bit */
#define XGP_MDIO_CLOCK_DIV_MASK 0x3F /* clock divide bits */
#define XGP_MDIO_CLOCK_DIV_100MHz  0x28    /* 100 MHz host clock */
//#define XGP_MDIO_CLOCK_DIV_100MHz  0x18    /* 100 MHz host clock */
/* @} */

/***************** Macros (Inline Functions) Definitions *********************/
/* necessary for pre-processor */
#define stringify(s)    tostring(s)
#define tostring(s)     #s

#define mtdcr(rn, v)    __asm__ __volatile__(\
                          "mtdcr " stringify(rn) ",%0\n"\
                          : : "r" (v)\
                        )

#define mfdcr(rn)       ({unsigned int rval; \
                          __asm__ __volatile__(\
                            "mfdcr %0," stringify(rn) "\n"\
                            : "=r" (rval)\
                          );\
                          rval;\
                        })

/** @name macros for reading/writing to GEMAC/PCS registers
 * @{
 */
#define XGPEmac_mReadStatsReg(BaseAddress, EmacNum, RegOffset, stats_msw, stats_lsw) ({ \
          mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
                 ((EmacNum) << 10) | (RegOffset)); \
          while ( !(mfdcr((BaseAddress) + XGP_HIF_RDY_STATUS_OFFSET )  \
                    & (XGP_DCR_RDYSTAT_STATS0_READ_MASK << (8*(EmacNum))))); \
          *(stats_lsw) = mfdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET); \
          *(stats_msw) = mfdcr((BaseAddress) + XGP_HIF_DATA_REG_MSW_OFFSET); })

#define XGPEmac_mReadAFReg(BaseAddress, EmacNum, RegOffset) ({ \
          mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
                 (EmacNum << 10) | (RegOffset)); \
          while ( !(mfdcr(BaseAddress + XGP_HIF_RDY_STATUS_OFFSET )  \
                    & (XGP_DCR_RDYSTAT_AF0_READ_MASK << (8*(EmacNum))))); \
          mfdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET) ; })

#define XGPEmac_mReadConfigReg(BaseAddress, EmacNum, RegOffset) ({ \
          mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
                 (EmacNum << 10) | (RegOffset)); \
          while ( !(mfdcr(BaseAddress + XGP_HIF_RDY_STATUS_OFFSET )  \
                    & (XGP_DCR_RDYSTAT_CONFIG0_READ_MASK << (8*(EmacNum))))); \
          mfdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET) ; })

#define XGPEmac_mWriteAFReg(BaseAddress, EmacNum, RegOffset, Data) \
            mtdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET, (Data)); \
            mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
             XGP_DCR_CNTLREG_WRITE_ENABLE_MASK | (EmacNum <<10)|(RegOffset) );\
          while ( !(mfdcr((BaseAddress) + XGP_HIF_RDY_STATUS_OFFSET )  \
                        & XGP_DCR_RDYSTAT_AF0_WRITE_MASK ))

#define XGPEmac_mWriteConfigReg(BaseAddress, EmacNum, RegOffset, Data) \
            mtdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET, (Data)); \
            mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
             XGP_DCR_CNTLREG_WRITE_ENABLE_MASK | (EmacNum <<10) | (RegOffset));\
          while ( !(mfdcr((BaseAddress) + XGP_HIF_RDY_STATUS_OFFSET )  \
                        & XGP_DCR_RDYSTAT_CONFIG0_WRITE_MASK ))
#if 0
#define XGPEmac_mSetControlReg(BaseAddress, Data) \
            mtdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET, (Data)); \
            mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
             XGP_DCR_CNTLREG_WRITE_ENABLE_MASK | \
             XGP_E0_MGMTCONFIG )
#endif

#define XGPEmac_mSetPcsReg(BaseAddress, EmacNum, PhyNum, Phyreg, Data) \
          mtdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET, (Data)); \
          mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
                XGP_DCR_CNTLREG_WRITE_ENABLE_MASK | XGP_MIIMWRDATA ); \
          mtdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET, \
             ((PhyNum << 5) | (Phyreg))); \
          mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET,  \
            XGP_DCR_CNTLREG_WRITE_ENABLE_MASK |  \
            ((EmacNum) << 10) | XGP_MIIMCNTL );   \
          while ( !(mfdcr((BaseAddress) + XGP_HIF_RDY_STATUS_OFFSET )  \
                        & XGP_DCR_RDYSTAT_MIIM0_WRITE_MASK)); \

#if 0
#define XGPEmac_Periph_mReadPcsReg(BaseAddress, EmacNum, PhyNum, Phyreg) \
          mtdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET, \
             ((PhyNum << 5) | (Phyreg))); \
          mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, \
            ((EmacNum) << 10) | XGP_MIIMCNTL ); \
          mfdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET )
#endif

#define XGPEmac_mGetPcsReg(BaseAddress, EmacNum, PhyNum, Phyreg) ({ \
          mtdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET,  \
             ((PhyNum << 5) | (Phyreg))); \
          mtdcr((BaseAddress) + XGP_HIF_CNTL_REG_OFFSET, XGP_MIIMCNTL ); \
          while ( !(mfdcr((BaseAddress) + XGP_HIF_RDY_STATUS_OFFSET )  \
                        & XGP_DCR_RDYSTAT_MIIM0_READ_MASK)); \
          mfdcr((BaseAddress) + XGP_HIF_DATA_REG_LSW_OFFSET ) ; })

#define XGPEmac_mReadReg(BaseAddress, RegOffset) \
            mfdcr((BaseAddress) + (RegOffset))

#define XGPEmac_mWriteReg(BaseAddress, RegOffset, Data) \
            mtdcr((BaseAddress) + (RegOffset), (Data))


#define XGPemac_GetRecvStats(instance_p, regstat_p) \
({ \
   *(regstat_p) = mfdcr((0x36)); \
})

#define XGPemac_ClearRecvStats(instance_p) \
({ \
   mtdcr(0x36, 0); \
})

/* @} */

#endif

